@@ -34,6 +34,9 @@
 #include <errno.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#if _MSC_VER
+#include <io.h>
+#endif
 #endif /*_WIN32_WCE*/
 
 
@@ -76,13 +79,13 @@ LpSection *lp_section_new(const char *name){
 
 void lp_item_destroy(void *pitem){
 	LpItem *item=(LpItem*)pitem;
-	free(item->key);
-	free(item->value);
+	ortp_free(item->key);
+	ortp_free(item->value);
 	free(item);
 }
 
 void lp_section_destroy(LpSection *sec){
-	free(sec->name);
+	ortp_free(sec->name);
 	ms_list_for_each(sec->items,lp_item_destroy);
 	ms_list_free(sec->items);
 	free(sec);
@@ -140,18 +143,20 @@ LpItem *lp_section_find_item(const LpSection *sec, const char *name){
 void lp_config_parse(LpConfig *lpconfig, FILE *file){
 	char tmp[MAX_LEN]= {'\0'};
 	LpSection *cur=NULL;
+	char *pos1,*pos2;
+	int nbs;
+	char secname[MAX_LEN];
+	char key[MAX_LEN];
+	LpItem *item;
 
 	if (file==NULL) return;
 
 	while(fgets(tmp,MAX_LEN,file)!=NULL){
 		tmp[sizeof(tmp) -1] = '\0';
-		char *pos1,*pos2;
 		pos1=strchr(tmp,'[');
 		if (pos1!=NULL && is_first_char(tmp,pos1) ){
 			pos2=strchr(pos1,']');
 			if (pos2!=NULL){
-				int nbs;
-				char secname[MAX_LEN];
 				secname[0]='\0';
 				/* found section */
 				*pos2='\0';
@@ -171,7 +176,6 @@ void lp_config_parse(LpConfig *lpconfig, FILE *file){
 		}else {
 			pos1=strchr(tmp,'=');
 			if (pos1!=NULL){
-				char key[MAX_LEN];
 				key[0]='\0';
 
 				*pos1='\0';
@@ -192,12 +196,12 @@ void lp_config_parse(LpConfig *lpconfig, FILE *file){
 						/* found a pair key,value */
 						
 						if (cur!=NULL){
-							LpItem *item=lp_section_find_item(cur,key);
+							item=lp_section_find_item(cur,key);
 							if (item==NULL){
 								lp_section_add_item(cur,lp_item_new(key,pos1));
 							}else{
-								ms_free(item->value);
-								item->value=strdup(pos1);
+								ortp_free(item->value);
+								item->value=ortp_strdup(pos1);
 							}
 							/*ms_message("Found %s=%s",key,pos1);*/
 						}else{
@@ -216,23 +220,26 @@ LpConfig * lp_config_new(const char *filename){
 
 LpConfig *lp_config_new_with_factory(const char *config_filename, const char *factory_config_filename) {
 	LpConfig *lpconfig=lp_new0(LpConfig,1);
+	
 	if (config_filename!=NULL){
 		ms_message("Using (r/w) config information from %s", config_filename);
 		lpconfig->filename=ortp_strdup(config_filename);
 		lpconfig->file=fopen(config_filename,"r+");
 		if (lpconfig->file!=NULL){
-			struct stat fileStat;
 			lp_config_parse(lpconfig,lpconfig->file);
 			fclose(lpconfig->file);
-#if !defined(_WIN32_WCE)
-			if ((stat(config_filename,&fileStat) == 0) && (S_ISREG(fileStat.st_mode))) {
-				/* make existing configuration files non-group/world-accessible */
-				if (chmod(config_filename, S_IRUSR | S_IWUSR) == -1) {
-					ms_warning("unable to correct permissions on "
-					"configuration file: %s", strerror(errno));
+#if !defined(WIN32)
+			{
+				struct stat fileStat;
+				if ((stat(config_filename,&fileStat) == 0) && (S_ISREG(fileStat.st_mode))) {
+					/* make existing configuration files non-group/world-accessible */
+					if (chmod(config_filename, S_IRUSR | S_IWUSR) == -1) {
+						ms_warning("unable to correct permissions on "
+							"configuration file: %s", strerror(errno));
+					}
 				}
 			}
-#endif /*_WIN32_WCE*/
+#endif /*WIN32*/
 			lpconfig->file=NULL;
 			lpconfig->modified=0;
 		}
@@ -256,13 +263,13 @@ int lp_config_read_file(LpConfig *lpconfig, const char *filename){
 }
 
 void lp_item_set_value(LpItem *item, const char *value){
-	free(item->value);
+	ortp_free(item->value);
 	item->value=ortp_strdup(value);
 }
 
 
 void lp_config_destroy(LpConfig *lpconfig){
-	if (lpconfig->filename!=NULL) free(lpconfig->filename);
+	if (lpconfig->filename!=NULL) ortp_free(lpconfig->filename);
 	ms_list_for_each(lpconfig->sections,(void (*)(void*))lp_section_destroy);
 	ms_list_free(lpconfig->sections);
 	free(lpconfig);
