@@ -24,9 +24,13 @@
  
 #ifndef LPCONFIG_H
 #define LPCONFIG_H
-
+#include <mediastreamer2/mscommon.h>
 #include <ortp/port.h>
 
+#ifndef LINPHONE_PUBLIC
+	#define LINPHONE_PUBLIC MS2_PUBLIC
+#endif
+
 /**
  * The LpConfig object is used to manipulate a configuration file.
  * 
@@ -72,7 +76,7 @@ extern "C" {
  * @param filename the filename of the config file to read to fill the instantiated LpConfig
  * @see lp_config_new_with_factory
  */
-LpConfig * lp_config_new(const char *filename);
+LINPHONE_PUBLIC LpConfig * lp_config_new(const char *filename);
 
 /**
  * Instantiates a LpConfig object from a user config file and a factory config file.
@@ -86,17 +90,24 @@ LpConfig * lp_config_new(const char *filename);
  * Therefore the configuration parameters defined in the user config file will be overwritten by the parameters
  * defined in the factory config file.
  */
-LpConfig * lp_config_new_with_factory(const char *config_filename, const char *factory_config_filename);
+LINPHONE_PUBLIC LpConfig * lp_config_new_with_factory(const char *config_filename, const char *factory_config_filename);
+
+/**
+ * Reads a user config file and fill the LpConfig with the read config values.
+ * @ingroup misc
+ * @param lpconfig The LpConfig object to fill with the content of the file
+ * @param filename The filename of the config file to read to fill the LpConfig
+ */
+LINPHONE_PUBLIC int lp_config_read_file(LpConfig *lpconfig, const char *filename);
 
-int lp_config_read_file(LpConfig *lpconfig, const char *filename);
 /**
  * Retrieves a configuration item as a string, given its section, key, and default value.
  * 
  * @ingroup misc
  * The default value string is returned if the config item isn't found.
 **/
-const char *lp_config_get_string(const LpConfig *lpconfig, const char *section, const char *key, const char *default_string);
-int lp_config_read_file(LpConfig *lpconfig, const char *filename);
+LINPHONE_PUBLIC const char *lp_config_get_string(const LpConfig *lpconfig, const char *section, const char *key, const char *default_string);
+
 /**
  * Retrieves a configuration item as a range, given its section, key, and default min and max values.
  *
@@ -104,14 +115,15 @@ int lp_config_read_file(LpConfig *lpconfig, const char *filename);
  * @return TRUE if the value is successfully parsed as a range, FALSE otherwise.
  * If FALSE is returned, min and max are filled respectively with default_min and default_max values.
  */
-bool_t lp_config_get_range(const LpConfig *lpconfig, const char *section, const char *key, int *min, int *max, int default_min, int default_max);
+LINPHONE_PUBLIC bool_t lp_config_get_range(const LpConfig *lpconfig, const char *section, const char *key, int *min, int *max, int default_min, int default_max);
+
 /**
  * Retrieves a configuration item as an integer, given its section, key, and default value.
  * 
  * @ingroup misc
  * The default integer value is returned if the config item isn't found.
 **/
-int lp_config_get_int(const LpConfig *lpconfig,const char *section, const char *key, int default_value);
+LINPHONE_PUBLIC int lp_config_get_int(const LpConfig *lpconfig,const char *section, const char *key, int default_value);
 
 /**
  * Retrieves a configuration item as a 64 bit integer, given its section, key, and default value.
@@ -119,80 +131,86 @@ int lp_config_get_int(const LpConfig *lpconfig,const char *section, const char *
  * @ingroup misc
  * The default integer value is returned if the config item isn't found.
 **/
-int64_t lp_config_get_int64(const LpConfig *lpconfig,const char *section, const char *key, int64_t default_value);
+LINPHONE_PUBLIC int64_t lp_config_get_int64(const LpConfig *lpconfig,const char *section, const char *key, int64_t default_value);
 
-
-int lp_config_read_file(LpConfig *lpconfig, const char *filename);
 /**
  * Retrieves a configuration item as a float, given its section, key, and default value.
  * 
  * @ingroup misc
  * The default float value is returned if the config item isn't found.
 **/
-float lp_config_get_float(const LpConfig *lpconfig,const char *section, const char *key, float default_value);
+LINPHONE_PUBLIC float lp_config_get_float(const LpConfig *lpconfig,const char *section, const char *key, float default_value);
+
 /**
  * Sets a string config item 
  *
  * @ingroup misc
 **/
-void lp_config_set_string(LpConfig *lpconfig,const char *section, const char *key, const char *value);
+LINPHONE_PUBLIC void lp_config_set_string(LpConfig *lpconfig,const char *section, const char *key, const char *value);
+
 /**
  * Sets a range config item
  *
  * @ingroup misc
  */
-void lp_config_set_range(LpConfig *lpconfig, const char *section, const char *key, int min_value, int max_value);
+LINPHONE_PUBLIC void lp_config_set_range(LpConfig *lpconfig, const char *section, const char *key, int min_value, int max_value);
+
 /**
  * Sets an integer config item
  *
  * @ingroup misc
 **/
-void lp_config_set_int(LpConfig *lpconfig,const char *section, const char *key, int value);
+LINPHONE_PUBLIC void lp_config_set_int(LpConfig *lpconfig,const char *section, const char *key, int value);
 
 /**
  * Sets an integer config item, but store it as hexadecimal
  *
  * @ingroup misc
 **/
-void lp_config_set_int_hex(LpConfig *lpconfig,const char *section, const char *key, int value);
+LINPHONE_PUBLIC void lp_config_set_int_hex(LpConfig *lpconfig,const char *section, const char *key, int value);
 
 /**
  * Sets a 64 bits integer config item
  *
  * @ingroup misc
 **/
-void lp_config_set_int64(LpConfig *lpconfig,const char *section, const char *key, int64_t value);
+LINPHONE_PUBLIC void lp_config_set_int64(LpConfig *lpconfig,const char *section, const char *key, int64_t value);
 
 /**
  * Sets a float config item
  *
  * @ingroup misc
 **/
-void lp_config_set_float(LpConfig *lpconfig,const char *section, const char *key, float value);	
+LINPHONE_PUBLIC void lp_config_set_float(LpConfig *lpconfig,const char *section, const char *key, float value);	
+
 /**
  * Writes the config file to disk.
  * 
  * @ingroup misc
 **/
-int lp_config_sync(LpConfig *lpconfig);
+LINPHONE_PUBLIC int lp_config_sync(LpConfig *lpconfig);
+
 /**
  * Returns 1 if a given section is present in the configuration.
  *
  * @ingroup misc
 **/
-int lp_config_has_section(const LpConfig *lpconfig, const char *section);
+LINPHONE_PUBLIC int lp_config_has_section(const LpConfig *lpconfig, const char *section);
+
 /**
  * Removes every pair of key,value in a section and remove the section.
  *
  * @ingroup misc
 **/
-void lp_config_clean_section(LpConfig *lpconfig, const char *section);
+LINPHONE_PUBLIC void lp_config_clean_section(LpConfig *lpconfig, const char *section);
+
 /**
  * Call a function for each section present in the configuration.
  *
  * @ingroup misc
 **/
 void lp_config_for_each_section(const LpConfig *lpconfig, void (*callback)(const char *section, void *ctx), void *ctx);
+
 /**
  * Call a function for each entry present in a section configuration.
  *
@@ -202,7 +220,7 @@ void lp_config_for_each_entry(const LpConfig *lpconfig, const char *section, voi
 
 /*tells whether uncommited (with lp_config_sync()) modifications exist*/
 int lp_config_needs_commit(const LpConfig *lpconfig);
-void lp_config_destroy(LpConfig *cfg);
+LINPHONE_PUBLIC void lp_config_destroy(LpConfig *cfg);
 	
 #ifdef __cplusplus
 }
