@@ -32,7 +32,11 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include <errno.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#if _MSC_VER
+#include <io.h>
+#else
 #include <unistd.h>
+#endif
 #include <fcntl.h>
 #endif /*_WIN32_WCE*/
 
@@ -44,133 +48,12 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include <ifaddrs.h>
 #endif
 #include <math.h>
-
-#if !defined(WIN32)
-
-static char lock_name[80];
-static char lock_set=0;
-/* put a lock file in /tmp. this is called when linphone runs as a daemon*/
-int set_lock_file()
-{
-	FILE *lockfile;
-
-	snprintf(lock_name,80,"/tmp/linphone.%i",getuid());
-	lockfile=fopen(lock_name,"w");
-	if (lockfile==NULL)
-	{
-		printf("Failed to create lock file.\n");
-		return(-1);
-	}
-	fprintf(lockfile,"%i",getpid());
-	fclose(lockfile);
-	lock_set=1;
-	return(0);
-}
-
-/* looks if there is a lock file. If presents return its content (the pid of the already running linphone), if not found, returns -1*/
-int get_lock_file()
-{
-	int pid;
-	FILE *lockfile;
-
-	snprintf(lock_name,80,"/tmp/linphone.%i",getuid());
-	lockfile=fopen(lock_name,"r");
-	if (lockfile==NULL)
-		return(-1);
-	if (fscanf(lockfile,"%i",&pid)!=1){
-		ms_warning("Could not read pid in lock file.");
-		fclose(lockfile);
-		return -1;
-	}
-	fclose(lockfile);
-	return pid;
-}
-
-/* remove the lock file if it was set*/
-int remove_lock_file()
-{
-	int err=0;
-	if (lock_set)
-	{
-		err=unlink(lock_name);
-		lock_set=0;
-	}
-	return(err);
-}
-
+#if _MSC_VER
+#define snprintf _snprintf
+#define popen _popen
+#define pclose _pclose
 #endif
 
-char *int2str(int number)
-{
-	char *numstr=ms_malloc(10);
-	snprintf(numstr,10,"%i",number);
-	return numstr;
-}
-
-void check_sound_device(LinphoneCore *lc)
-{
-#ifdef _linux
-	int fd=0;
-	int len;
-	int a;
-	char *file=NULL;
-	char *i810_audio=NULL;
-	char *snd_pcm_oss=NULL;
-	char *snd_mixer_oss=NULL;
-	char *snd_pcm=NULL;
-	fd=open("/proc/modules",O_RDONLY);
-
-	if (fd>0){
-		/* read the entire /proc/modules file and check if sound conf seems correct */
-		/*a=fstat(fd,&statbuf);
-		if (a<0) ms_warning("Can't stat /proc/modules:%s.",strerror(errno));
-		len=statbuf.st_size;
-		if (len==0) ms_warning("/proc/modules has zero size!");
-		*/
-		/***** fstat does not work on /proc/modules for unknown reason *****/
-		len=6000;
-		file=ms_malloc(len+1);
-		a=read(fd,file,len);
-		if (a<len) file=ms_realloc(file,a+1);
-		file[a]='\0';
-		i810_audio=strstr(file,"i810_audio");
-		if (i810_audio!=NULL){
-			/* I'm sorry i put this warning in comments because
-			 * i don't use yet the right driver !! */
-/*			lc->vtable.display_warning(lc,_("You are currently using the i810_audio driver.\nThis driver is buggy and so does not work with Linphone.\nWe suggest that you replace it by its equivalent ALSA driver,\neither with packages from your distribution, or by downloading\nALSA drivers at http://www.alsa-project.org."));*/
-			goto end;
-		}
-		snd_pcm=strstr(file,"snd-pcm");
-		if (snd_pcm!=NULL){
-			snd_pcm_oss=strstr(file,"snd-pcm-oss");
-			snd_mixer_oss=strstr(file,"snd-mixer-oss");
-			if (snd_pcm_oss==NULL){
-				lc->vtable.display_warning(lc,_("Your computer appears to be using ALSA sound drivers.\nThis is the best choice. However the pcm oss emulation module\nis missing and linphone needs it. Please execute\n'modprobe snd-pcm-oss' as root to load it."));
-			}
-			if (snd_mixer_oss==NULL){
-				lc->vtable.display_warning(lc,_("Your computer appears to be using ALSA sound drivers.\nThis is the best choice. However the mixer oss emulation module\nis missing and linphone needs it. Please execute\n 'modprobe snd-mixer-oss' as root to load it."));
-			}
-		}
-	}else {
-
-		ms_warning("Could not open /proc/modules.");
-	}
-	/* now check general volume. Some user forget to rise it and then complain that linphone is
-	not working */
-	/* but some other users complain that linphone should not change levels...
-	if (lc->sound_conf.sndcard!=NULL){
-		a=snd_card_get_level(lc->sound_conf.sndcard,SND_CARD_LEVEL_GENERAL);
-		if (a<50){
-			ms_warning("General level is quite low (%i). Linphone rises it up for you.",a);
-			snd_card_set_level(lc->sound_conf.sndcard,SND_CARD_LEVEL_GENERAL,80);
-		}
-	}
-	*/
-	end:
-	if (file!=NULL) ms_free(file);
-	if (fd>0) close(fd);
-#endif
-}
 
 #define UDP_HDR_SZ 8
 #define RTP_HDR_SZ 12
@@ -415,31 +298,38 @@ static int sendStunRequest(int sock, const struct sockaddr *server, socklen_t ad
 	return 0;
 }
 
-int parse_hostname_to_addr(const char *server, struct sockaddr_storage *ss, socklen_t *socklen){
-	struct addrinfo hints,*res=NULL;
-	int family = PF_INET;
-	int port_int = 3478;
-	int ret;
-	char port[6];
-	char host[NI_MAXHOST];
+int linphone_parse_host_port(const char *input, char *host, size_t hostlen, int *port){
+	char tmphost[NI_MAXHOST]={0};
 	char *p1, *p2;
-	if ((sscanf(server, "[%64[^]]]:%d", host, &port_int) == 2) || (sscanf(server, "[%64[^]]]", host) == 1)) {
-		family = PF_INET6;
+	
+	if ((sscanf(input, "[%64[^]]]:%d", tmphost, port) == 2) || (sscanf(input, "[%64[^]]]", tmphost) == 1)) {
+		
 	} else {
-		p1 = strchr(server, ':');
-		p2 = strrchr(server, ':');
-		if (p1 && p2 && (p1 != p2)) {
-			family = PF_INET6;
-			host[NI_MAXHOST-1]='\0';
-			strncpy(host, server, sizeof(host) - 1);
-		} else if (sscanf(server, "%[^:]:%d", host, &port_int) != 2) {
-			host[NI_MAXHOST-1]='\0';
-			strncpy(host, server, sizeof(host) - 1);
+		p1 = strchr(input, ':');
+		p2 = strrchr(input, ':');
+		if (p1 && p2 && (p1 != p2)) {/* an ipv6 address without port*/
+			strncpy(tmphost, input, sizeof(tmphost) - 1);
+		} else if (sscanf(input, "%[^:]:%d", tmphost, port) != 2) {
+			/*no port*/
+			strncpy(tmphost, input, sizeof(tmphost) - 1);
 		}
 	}
+	strncpy(host,tmphost,hostlen);
+	return 0;
+}
+
+int parse_hostname_to_addr(const char *server, struct sockaddr_storage *ss, socklen_t *socklen, int default_port){
+	struct addrinfo hints,*res=NULL;
+	char port[6];
+	char host[NI_MAXHOST];
+	int port_int=default_port;
+	int ret;
+	
+	linphone_parse_host_port(server,host,sizeof(host),&port_int);
+	
 	snprintf(port, sizeof(port), "%d", port_int);
 	memset(&hints,0,sizeof(hints));
-	hints.ai_family=family;
+	hints.ai_family=AF_UNSPEC;
 	hints.ai_socktype=SOCK_DGRAM;
 	hints.ai_protocol=IPPROTO_UDP;
 	ret=getaddrinfo(host,port,&hints,&res);
@@ -486,8 +376,7 @@ int linphone_core_run_stun_tests(LinphoneCore *lc, LinphoneCall *call){
 		return -1;
 	}
 	if (server!=NULL){
-		struct sockaddr_storage ss;
-		socklen_t ss_len;
+		const struct addrinfo *ai=linphone_core_get_stun_server_addrinfo(lc);
 		ortp_socket_t sock1=-1, sock2=-1;
 		int loops=0;
 		bool_t video_enabled=linphone_core_video_enabled(lc);
@@ -497,8 +386,8 @@ int linphone_core_run_stun_tests(LinphoneCore *lc, LinphoneCall *call){
 		double elapsed;
 		int ret=0;
 		
-		if (parse_hostname_to_addr(server,&ss,&ss_len)<0){
-			ms_error("Fail to parser stun server address: %s",server);
+		if (ai==NULL){
+			ms_error("Could not obtain stun server addrinfo.");
 			return -1;
 		}
 		if (lc->vtable.display_status!=NULL)
@@ -513,24 +402,20 @@ int linphone_core_run_stun_tests(LinphoneCore *lc, LinphoneCall *call){
 		}
 		got_audio=FALSE;
 		got_video=FALSE;
-		gettimeofday(&init,NULL);
+		ortp_gettimeofday(&init,NULL);
 		do{
 			
 			int id;
 			if (loops%20==0){
 				ms_message("Sending stun requests...");
-				sendStunRequest(sock1,(struct sockaddr*)&ss,ss_len,11,TRUE);
-				sendStunRequest(sock1,(struct sockaddr*)&ss,ss_len,1,FALSE);
+				sendStunRequest(sock1,ai->ai_addr,ai->ai_addrlen,11,TRUE);
+				sendStunRequest(sock1,ai->ai_addr,ai->ai_addrlen,1,FALSE);
 				if (sock2!=-1){
-					sendStunRequest(sock2,(struct sockaddr*)&ss,ss_len,22,TRUE);
-					sendStunRequest(sock2,(struct sockaddr*)&ss,ss_len,2,FALSE);
+					sendStunRequest(sock2,ai->ai_addr,ai->ai_addrlen,22,TRUE);
+					sendStunRequest(sock2,ai->ai_addr,ai->ai_addrlen,2,FALSE);
 				}
 			}
-#ifdef WIN32
-			Sleep(10);
-#else
-			usleep(10000);
-#endif
+			ms_usleep(10000);
 
 			if (recvStunResponse(sock1,ac->addr,
 						&ac->port,&id)>0){
@@ -550,7 +435,7 @@ int linphone_core_run_stun_tests(LinphoneCore *lc, LinphoneCall *call){
 					cone_video=TRUE;
 				got_video=TRUE;
 			}
-			gettimeofday(&cur,NULL);
+			ortp_gettimeofday(&cur,NULL);
 			elapsed=((cur.tv_sec-init.tv_sec)*1000.0) +  ((cur.tv_usec-init.tv_usec)/1000.0);
 			if (elapsed>2000)  {
 				ms_message("Stun responses timeout, going ahead.");
@@ -594,9 +479,10 @@ int linphone_core_get_edge_ptime(LinphoneCore *lc){
 }
 
 void linphone_core_adapt_to_network(LinphoneCore *lc, int ping_time_ms, LinphoneCallParams *params){
+	int threshold;
 	if (ping_time_ms>0 && lp_config_get_int(lc->config,"net","activate_edge_workarounds",0)==1){
 		ms_message("Stun server ping time is %i ms",ping_time_ms);
-		int threshold=lp_config_get_int(lc->config,"net","edge_ping_time",500);
+		threshold=lp_config_get_int(lc->config,"net","edge_ping_time",500);
 		
 		if (ping_time_ms>threshold){
 			/* we might be in a 2G network*/
@@ -610,13 +496,64 @@ void linphone_core_adapt_to_network(LinphoneCore *lc, int ping_time_ms, Linphone
 	}
 }
 
+static void stun_server_resolved(LinphoneCore *lc, const char *name, struct addrinfo *addrinfo){
+	if (lc->net_conf.stun_addrinfo){
+		freeaddrinfo(lc->net_conf.stun_addrinfo);
+		lc->net_conf.stun_addrinfo=NULL;
+	}
+	if (addrinfo){
+		ms_message("Stun server resolution successful.");
+	}else{
+		ms_warning("Stun server resolution failed.");
+	}
+	lc->net_conf.stun_addrinfo=addrinfo;
+	lc->net_conf.stun_res=NULL;
+}
 
+void linphone_core_resolve_stun_server(LinphoneCore *lc){
+	/*
+	 * WARNING: stun server resolution only done in IPv4.
+	 * TODO: use IPv6 resolution if linphone_core_ipv6_enabled()==TRUE and use V4Mapped addresses for ICE gathering.
+	 */
+	const char *server=lc->net_conf.stun_server;
+	if (lc->sal && server){
+		char host[NI_MAXHOST];
+		int port=3478;
+		linphone_parse_host_port(server,host,sizeof(host),&port);
+		lc->net_conf.stun_res=sal_resolve_a(lc->sal,host,port,AF_INET,(SalResolverCallback)stun_server_resolved,lc);
+	}
+}
+
+/*
+ * This function returns the addrinfo representation of the stun server address.
+ * It is critical not to block for a long time if it can't be resolved, otherwise this stucks the main thread when making a call.
+ * On the contrary, a fully asynchronous call initiation is complex to develop.
+ * The compromise is then:
+ * - have a cache of the stun server addrinfo
+ * - this cached value is returned when it is non-null
+ * - an asynchronous resolution is asked each time this function is called to ensure frequent refreshes of the cached value.
+ * - if no cached value exists, block for a short time; this case must be unprobable because the resolution will be asked each time the stun server value is 
+ * changed.
+**/
+const struct addrinfo *linphone_core_get_stun_server_addrinfo(LinphoneCore *lc){
+	const char *server=linphone_core_get_stun_server(lc);
+	if (server){
+		int wait_ms=0;
+		int wait_limit=1000;
+		linphone_core_resolve_stun_server(lc);
+		while (!lc->net_conf.stun_addrinfo && lc->net_conf.stun_res!=NULL && wait_ms<wait_limit){
+			sal_iterate(lc->sal);
+			ms_usleep(50000);
+			wait_ms+=50;
+		}
+	}
+	return lc->net_conf.stun_addrinfo;
+}
 
 int linphone_core_gather_ice_candidates(LinphoneCore *lc, LinphoneCall *call)
 {
 	char local_addr[64];
-	struct sockaddr_storage ss;
-	socklen_t ss_len;
+	const struct addrinfo *ai;
 	IceCheckList *audio_check_list;
 	IceCheckList *video_check_list;
 	const char *server = linphone_core_get_stun_server(lc);
@@ -626,20 +563,20 @@ int linphone_core_gather_ice_candidates(LinphoneCore *lc, LinphoneCall *call)
 	video_check_list = ice_session_check_list(call->ice_session, 1);
 	if (audio_check_list == NULL) return -1;
 
-	if (lc->sip_conf.ipv6_enabled){
-		ms_warning("stun support is not implemented for ipv6");
+	if (call->af==AF_INET6){
+		ms_warning("Ice gathering is not implemented for ipv6");
 		return -1;
 	}
-
-	if (parse_hostname_to_addr(server, &ss, &ss_len) < 0) {
-		ms_error("Fail to parser stun server address: %s", server);
+	ai=linphone_core_get_stun_server_addrinfo(lc);
+	if (ai==NULL){
+		ms_warning("Fail to resolve STUN server for ICE gathering.");
 		return -1;
 	}
 	if (lc->vtable.display_status != NULL)
 		lc->vtable.display_status(lc, _("ICE local candidates gathering in progress..."));
 
 	/* Gather local host candidates. */
-	if (linphone_core_get_local_ip_for(AF_INET, server, local_addr) < 0) {
+	if (linphone_core_get_local_ip_for(AF_INET, NULL, local_addr) < 0) {
 		ms_error("Fail to get local ip");
 		return -1;
 	}
@@ -657,7 +594,7 @@ int linphone_core_gather_ice_candidates(LinphoneCore *lc, LinphoneCall *call)
 
 	ms_message("ICE: gathering candidate from [%s]",server);
 	/* Gather local srflx candidates. */
-	ice_session_gather_candidates(call->ice_session, ss, ss_len);
+	ice_session_gather_candidates(call->ice_session, ai->ai_addr, ai->ai_addrlen);
 	return 0;
 }
 
@@ -903,6 +840,12 @@ void linphone_core_update_ice_from_remote_media_description(LinphoneCall *call,
 				ice_check_list_set_state(cl, ICL_Failed);
 			} else if (stream->rtp_port == 0) {
 				ice_session_remove_check_list(call->ice_session, cl);
+#ifdef VIDEO_ENABLED
+				if (stream->type==SalVideo && call->videostream){
+					video_stream_stop(call->videostream);
+					call->videostream=NULL;
+				}
+#endif
 			} else {
 				if ((stream->ice_pwd[0] != '\0') && (stream->ice_ufrag[0] != '\0'))
 					ice_check_list_set_remote_credentials(cl, stream->ice_ufrag, stream->ice_pwd);
@@ -1068,12 +1011,12 @@ static int get_local_ip_for_with_connect(int type, const char *dest, char *resul
 	socklen_t s;
 
 	memset(&hints,0,sizeof(hints));
-	hints.ai_family=(type==AF_INET6) ? PF_INET6 : PF_INET;
+	hints.ai_family=type;
 	hints.ai_socktype=SOCK_DGRAM;
 	/*hints.ai_flags=AI_NUMERICHOST|AI_CANONNAME;*/
 	err=getaddrinfo(dest,"5060",&hints,&res);
 	if (err!=0){
-		ms_error("getaddrinfo() error: %s",gai_strerror(err));
+		ms_error("getaddrinfo() error for %s : %s",dest, gai_strerror(err));
 		return -1;
 	}
 	if (res==NULL){
@@ -1088,7 +1031,8 @@ static int get_local_ip_for_with_connect(int type, const char *dest, char *resul
 	}
 	err=connect(sock,res->ai_addr,res->ai_addrlen);
 	if (err<0) {
-		ms_error("Error in connect: %s",strerror(errno));
+		/*the network isn't reachable*/
+		if (getSocketErrorCode()!=ENETUNREACH) ms_error("Error in connect: %s",strerror(errno));
  		freeaddrinfo(res);
  		close_socket(sock);
 		return -1;
@@ -1114,7 +1058,8 @@ static int get_local_ip_for_with_connect(int type, const char *dest, char *resul
 		ms_error("getnameinfo error: %s",strerror(errno));
 	}
 	close_socket(sock);
-	ms_message("Local interface to reach %s is %s.",dest,result);
+
+
 	return 0;
 }
 
@@ -1134,7 +1079,6 @@ int linphone_core_get_local_ip_for(int type, const char *dest, char *result){
 	 * try to find 'the' running interface with getifaddrs*/
 	
 #ifdef HAVE_GETIFADDRS
-
 	/*we use getifaddrs for lookup of default interface */
 	int found_ifs;
 
@@ -1149,26 +1093,186 @@ int linphone_core_get_local_ip_for(int type, const char *dest, char *result){
       return 0;  
 }
 
-#ifndef WIN32
-#include <resolv.h>
-
-
-
-void _linphone_core_configure_resolver(){
-/*bionic declares _res but does not define nor export it !!*/
-#ifdef ANDROID
-	/*timeout and attempts are the same as retrans and retry, but are android specific names.*/
-	setenv("RES_OPTIONS","timeout:2 attempts:2 retrans:2 retry:2",1);
-#else
-	res_init();
-	_res.retrans=2; /*retransmit every two seconds*/
-	_res.retry=2; /*only two times per DNS server*/
-#endif
+SalReason linphone_reason_to_sal(LinphoneReason reason){
+	switch(reason){
+		case LinphoneReasonNone:
+			return SalReasonUnknown;
+		case LinphoneReasonNoResponse:
+			return SalReasonUnknown;
+		case LinphoneReasonBadCredentials:
+			return SalReasonForbidden;
+		case LinphoneReasonDeclined:
+			return SalReasonDeclined;
+		case LinphoneReasonNotFound:
+			return SalReasonNotFound;
+		case LinphoneReasonNotAnswered:
+			return SalReasonTemporarilyUnavailable;
+		case LinphoneReasonBusy:
+			return SalReasonBusy;
+		case LinphoneReasonNotAcceptable:
+			return SalReasonNotAcceptable;
+		case LinphoneReasonIOError:
+			return SalReasonServiceUnavailable;
+		case LinphoneReasonDoNotDisturb:
+			return SalReasonDoNotDisturb;
+		case LinphoneReasonUnauthorized:
+			return SalReasonUnauthorized;
+		case LinphoneReasonUnsupportedContent:
+			return SalReasonUnsupportedContent;
+		case LinphoneReasonNoMatch:
+			return SalReasonNoMatch;
+	}
+	return SalReasonUnknown;
 }
 
-#else
-
-void _linphone_core_configure_resolver(){
+LinphoneReason linphone_reason_from_sal(SalReason r){
+	LinphoneReason ret=LinphoneReasonNone;
+	switch(r){
+		case SalReasonUnknown:
+			ret=LinphoneReasonNone;
+			break;
+		case SalReasonBusy:
+			ret=LinphoneReasonBusy;
+			break;
+		case SalReasonDeclined:
+			ret=LinphoneReasonDeclined;
+			break;
+		case SalReasonDoNotDisturb:
+			ret=LinphoneReasonDoNotDisturb;
+			break;
+		case SalReasonForbidden:
+			ret=LinphoneReasonBadCredentials;
+			break;
+		case SalReasonNotAcceptable:
+			ret=LinphoneReasonNotAcceptable;
+			break;
+		case SalReasonNotFound:
+			ret=LinphoneReasonNotFound;
+			break;
+		case SalReasonRedirect:
+			ret=LinphoneReasonNone;
+			break;
+		case SalReasonTemporarilyUnavailable:
+			ret=LinphoneReasonNone;
+			break;
+		case SalReasonServiceUnavailable:
+			ret=LinphoneReasonIOError;
+			break;
+		case SalReasonRequestPending:
+			ret=LinphoneReasonNone;
+			break;
+		case SalReasonUnauthorized:
+			ret=LinphoneReasonUnauthorized;
+			break;
+		case SalReasonUnsupportedContent:
+			ret=LinphoneReasonUnsupportedContent;
+		break;
+		case SalReasonNoMatch:
+			ret=LinphoneReasonNoMatch;
+		break;
+	}
+	return ret;
 }
 
-#endif
+/**
+ * Set the name of the mediastreamer2 filter to be used for rendering video.
+ * This is for advanced users of the library, mainly to workaround hardware/driver bugs.
+ * @ingroup media_parameters
+**/
+void linphone_core_set_video_display_filter(LinphoneCore *lc, const char *filter_name){
+	lp_config_set_string(lc->config,"video","displaytype",filter_name);
+}
+
+/**
+ * Get the name of the mediastreamer2 filter used for rendering video.
+ * @ingroup media_parameters
+**/
+const char *linphone_core_get_video_display_filter(LinphoneCore *lc){
+	return lp_config_get_string(lc->config,"video","displaytype",NULL);
+}
+
+/**
+ * Queue a task into the main loop. The data pointer must remain valid until the task is completed.
+ * task_fun must return BELLE_SIP_STOP when job is finished.
+**/
+void linphone_core_queue_task(LinphoneCore *lc, belle_sip_source_func_t task_fun, void *data, const char *task_description){
+	belle_sip_source_t *s=sal_create_timer(lc->sal,task_fun,data, 20, task_description);
+	belle_sip_object_unref(s);
+}
+
+static int get_unique_transport(LinphoneCore *lc, LinphoneTransportType *type, int *port){
+	LCSipTransports tp;
+	linphone_core_get_sip_transports(lc,&tp);
+	if (tp.tcp_port==0 && tp.tls_port==0 && tp.udp_port!=0){
+		*type=LinphoneTransportUdp;
+		*port=tp.udp_port;
+		return 0;
+	}else if (tp.tcp_port==0 && tp.udp_port==0 && tp.tls_port!=0){
+		*type=LinphoneTransportTls;
+		*port=tp.tls_port;
+		return 0;
+	}else if (tp.tcp_port!=0 && tp.udp_port==0 && tp.tls_port==0){
+		*type=LinphoneTransportTcp;
+		*port=tp.tcp_port;
+		return 0;
+	}
+	return -1;
+}
+
+static void linphone_core_migrate_proxy_config(LinphoneCore *lc, LinphoneTransportType type){
+	const MSList *elem;
+	for(elem=linphone_core_get_proxy_config_list(lc);elem!=NULL;elem=elem->next){
+		LinphoneProxyConfig *cfg=(LinphoneProxyConfig*)elem->data;
+		const char *proxy=linphone_proxy_config_get_addr(cfg);
+		const char *route=linphone_proxy_config_get_route(cfg);
+		LinphoneAddress *proxy_addr=linphone_address_new(proxy);
+		LinphoneAddress *route_addr=NULL;
+		char *tmp;
+		if (route) route_addr=linphone_address_new(route);
+		if (proxy_addr){
+			linphone_address_set_transport(proxy_addr,type);
+			tmp=linphone_address_as_string(proxy_addr);
+			linphone_proxy_config_set_server_addr(cfg,tmp);
+			ms_free(tmp);
+			linphone_address_destroy(proxy_addr);
+		}
+		if (route_addr){
+			linphone_address_set_transport(route_addr,type);
+			tmp=linphone_address_as_string(route_addr);
+			linphone_proxy_config_set_route(cfg,tmp);
+			ms_free(tmp);
+			linphone_address_destroy(route_addr);
+		}
+	}
+}
+
+/**
+ * Migrate configuration so that all SIP transports are enabled.
+ * Versions of linphone < 3.7 did not support using multiple SIP transport simultaneously.
+ * This function helps application to migrate the configuration so that all transports are enabled.
+ * Existing proxy configuration are added a transport parameter so that they continue using the unique transport that was set previously.
+ * This function must be used just after creating the core, before any call to linphone_core_iterate()
+ * @param lc the linphone core
+ * @returns 1 if migration was done, 0 if not done because unnecessary or already done, -1 in case of error.
+ * @ingroup initializing
+**/
+int linphone_core_migrate_to_multi_transport(LinphoneCore *lc){
+	if (!lp_config_get_int(lc->config,"sip","multi_transport_migration_done",0)){
+		LinphoneTransportType tpt;
+		int port;
+		if (get_unique_transport(lc,&tpt,&port)==0){
+			LCSipTransports newtp={0};
+			ms_message("Core is using a single SIP transport, migrating proxy config and enabling multi-transport.");
+			linphone_core_migrate_proxy_config(lc,tpt);
+			newtp.udp_port=port;
+			newtp.tcp_port=port;
+			newtp.tls_port=LC_SIP_TRANSPORT_RANDOM;
+			linphone_core_set_sip_transports(lc,&newtp);
+		}
+		lp_config_set_int(lc->config,"sip","multi_transport_migration_done",1);
+		return 1;
+	}
+	return 0;
+}
+
+
