@@ -86,7 +86,8 @@ struct _UpnpContext {
 
 
 bool_t linphone_core_upnp_hook(void *data);
-void linphone_core_upnp_refresh(UpnpContext *ctx);
+void linphone_upnp_update(UpnpContext *ctx);
+bool_t linphone_upnp_is_blacklisted(UpnpContext *ctx);
 
 UpnpPortBinding *linphone_upnp_port_binding_new();
 UpnpPortBinding *linphone_upnp_port_binding_new_with_parameters(upnp_igd_ip_protocol protocol, int local_port, int external_port);
@@ -198,6 +199,7 @@ void linphone_upnp_igd_callback(void *cookie, upnp_igd_event event, void *arg) {
 	const char *ip_address = NULL;
 	const char *connection_status = NULL;
 	bool_t nat_enabled = FALSE;
+	bool_t blacklisted = FALSE;
 	LinphoneUpnpState old_state;
 
 	if(lupnp == NULL || lupnp->upnp_igd_ctxt == NULL) {
@@ -217,6 +219,7 @@ void linphone_upnp_igd_callback(void *cookie, upnp_igd_event event, void *arg) {
 		ip_address = upnp_igd_get_external_ipaddress(lupnp->upnp_igd_ctxt);
 		connection_status = upnp_igd_get_connection_status(lupnp->upnp_igd_ctxt);
 		nat_enabled = upnp_igd_get_nat_enabled(lupnp->upnp_igd_ctxt);
+		blacklisted = linphone_upnp_is_blacklisted(lupnp);
 
 		if(ip_address == NULL || connection_status == NULL) {
 			ms_message("uPnP IGD: Pending");
@@ -224,11 +227,14 @@ void linphone_upnp_igd_callback(void *cookie, upnp_igd_event event, void *arg) {
 		} else if(strcasecmp(connection_status, "Connected")  || !nat_enabled) {
 			ms_message("uPnP IGD: Not Available");
 			lupnp->state = LinphoneUpnpStateNotAvailable;
+		} else if(blacklisted) {
+			ms_message("uPnP IGD: Router is blacklisted");
+			lupnp->state = LinphoneUpnpStateBlacklisted;
 		} else {
 			ms_message("uPnP IGD: Connected");
 			lupnp->state = LinphoneUpnpStateOk;
 			if(old_state != LinphoneUpnpStateOk) {
-				linphone_core_upnp_refresh(lupnp);
+				linphone_upnp_update(lupnp);
 			}
 		}
 
@@ -316,7 +322,12 @@ void linphone_upnp_igd_callback(void *cookie, upnp_igd_event event, void *arg) {
 
 UpnpContext* linphone_upnp_context_new(LinphoneCore *lc) {
 	UpnpContext *lupnp = (UpnpContext *)ms_new0(UpnpContext,1);
-
+	char address[LINPHONE_IPADDR_SIZE];
+	const char*upnp_binding_address=address;
+	if (linphone_core_get_local_ip_for(lc->sip_conf.ipv6_enabled ? AF_INET6 : AF_INET,NULL,address)) {
+		ms_warning("Linphone core [%p] cannot guess local address for upnp, let's choice the lib",lc);
+		upnp_binding_address=NULL;
+	}
 	ms_mutex_init(&lupnp->mutex, NULL);
 	ms_cond_init(&lupnp->empty_cond, NULL);
 
@@ -328,7 +339,7 @@ UpnpContext* linphone_upnp_context_new(LinphoneCore *lc) {
 	lupnp->adding_configs = NULL;
 	lupnp->removing_configs = NULL;
 	lupnp->state = LinphoneUpnpStateIdle;
-	ms_message("uPnP IGD: New %p for core %p", lupnp, lc);
+	ms_message("uPnP IGD: New %p for core %p bound to %s", lupnp, lc,upnp_binding_address);
 
 	// Init ports
 	lupnp->sip_udp = NULL;
@@ -338,7 +349,7 @@ UpnpContext* linphone_upnp_context_new(LinphoneCore *lc) {
 	linphone_core_add_iterate_hook(lc, linphone_core_upnp_hook, lupnp);
 
 	lupnp->upnp_igd_ctxt = NULL;
-	lupnp->upnp_igd_ctxt = upnp_igd_create(linphone_upnp_igd_callback, linphone_upnp_igd_print, NULL, lupnp);
+	lupnp->upnp_igd_ctxt = upnp_igd_create(linphone_upnp_igd_callback, linphone_upnp_igd_print, address, lupnp);
 	if(lupnp->upnp_igd_ctxt == NULL) {
 		lupnp->state = LinphoneUpnpStateKo;
 		ms_error("Can't create uPnP IGD context");
@@ -495,6 +506,45 @@ int linphone_upnp_context_get_external_port(UpnpContext *lupnp) {
 	return port;
 }
 
+bool_t linphone_upnp_is_blacklisted(UpnpContext *lupnp) {
+	const char * device_model_name = upnp_igd_get_device_model_name(lupnp->upnp_igd_ctxt);
+	const char * device_model_number = upnp_igd_get_device_model_number(lupnp->upnp_igd_ctxt); 	
+	const char * blacklist = lp_config_get_string(lupnp->lc->config, "net", "upnp_blacklist", NULL);
+	bool_t blacklisted = FALSE;
+	char *str;
+	char *pch;
+	char *model_name;
+	char *model_number;
+
+	// Sanity checks
+	if(device_model_name == NULL || device_model_number == NULL || blacklist == NULL) {
+		return FALSE;
+	}
+
+	// Find in the list	
+	str = strdup(blacklist);
+	pch = strtok(str, ";");
+	while (pch != NULL && !blacklisted) {
+		// Extract model name & number
+		model_name = pch;
+		model_number = strstr(pch, ",");
+		if(model_number != NULL) {
+			*(model_number++) = '\0';
+		}
+
+		// Compare with current device
+		if(strcmp(model_name, device_model_name) == 0) {
+			if(model_number == NULL || strcmp(model_number, device_model_number) == 0) {
+				blacklisted = TRUE;
+			}
+		}	
+		pch = strtok(NULL, ";");
+	}
+	free(str);
+
+	return blacklisted;
+}
+
 void linphone_upnp_refresh(UpnpContext * lupnp) {
 	upnp_igd_refresh(lupnp->upnp_igd_ctxt);
 }
@@ -800,13 +850,24 @@ int linphone_upnp_call_process(LinphoneCall *call) {
 	return ret;
 }
 
-void linphone_core_upnp_refresh(UpnpContext *lupnp) {
+static const char *linphone_core_upnp_get_charptr_null(const char *str) {
+	if(str != NULL) {
+		return str;
+	}
+	return "(Null)";
+}
+
+void linphone_upnp_update(UpnpContext *lupnp) {
 	MSList *global_list = NULL;
 	MSList *list = NULL;
 	MSList *item;
 	LinphoneCall *call;
 	UpnpPortBinding *port_mapping, *port_mapping2;
 
+	ms_message("uPnP IGD: Name:%s", linphone_core_upnp_get_charptr_null(upnp_igd_get_device_name(lupnp->upnp_igd_ctxt)));
+	ms_message("uPnP IGD: Device:%s %s", 
+                   linphone_core_upnp_get_charptr_null(upnp_igd_get_device_model_name(lupnp->upnp_igd_ctxt)), 
+	           linphone_core_upnp_get_charptr_null(upnp_igd_get_device_model_number(lupnp->upnp_igd_ctxt)));
 	ms_message("uPnP IGD: Refresh mappings");
 
 	if(lupnp->sip_udp != NULL) {
