@@ -87,6 +87,18 @@ static GdkPixbuf *create_chat_picture(){
 	return pixbuf;
 }
 
+static GdkPixbuf *create_composing_unread_msg(){
+	GdkPixbuf *pixbuf;
+	pixbuf = create_pixbuf("composing_active_chat.png");
+	return pixbuf;
+}
+
+static GdkPixbuf *create_composing_chat_picture(){
+	GdkPixbuf *pixbuf;
+	pixbuf = create_pixbuf("composing_chat.png");
+	return pixbuf;
+}
+
 /*
 void linphone_gtk_set_friend_status(GtkWidget *friendlist , LinphoneFriend * fid, const gchar *url, const gchar *status, const gchar *img){
 	GtkTreeIter iter;
@@ -197,9 +209,9 @@ void linphone_gtk_delete_history(GtkWidget *button){
 		gtk_tree_model_get (model, &iter,FRIEND_CHATROOM , &cr, -1);
 		linphone_chat_room_delete_history(cr);
 		if(chat_view!=NULL){
-			char *from=g_object_get_data(G_OBJECT(friendlist),"from");
-			char *addr=linphone_address_as_string(linphone_friend_get_address(lf));
-			if(g_strcmp0(from,addr)==0){
+			const LinphoneAddress *from=linphone_gtk_friend_list_get_active_address();
+			const LinphoneAddress *addr=linphone_friend_get_address(lf);
+			if(linphone_address_weak_equal(from,addr)){
 				GtkTextView *text_view=GTK_TEXT_VIEW(linphone_gtk_get_widget(chat_view,"textview"));
 				GtkTextIter start;
 				GtkTextIter end;
@@ -227,15 +239,23 @@ void linphone_gtk_friend_list_update_chat_picture(){
 	GtkWidget *friendlist=linphone_gtk_get_widget(w,"contact_list");
 	GtkTreeModel *model=gtk_tree_view_get_model(GTK_TREE_VIEW(friendlist));
 	LinphoneChatRoom *cr=NULL;
+	bool_t is_composing;
 	int nbmsg=0;
 	if (gtk_tree_model_get_iter_first(model,&iter)) {
 		do{
 			gtk_tree_model_get (model, &iter,FRIEND_CHATROOM , &cr, -1);
 			nbmsg=linphone_chat_room_get_unread_messages_count(cr);
+			is_composing=linphone_chat_room_is_remote_composing(cr);
 			if(nbmsg != 0){
-				gtk_list_store_set(GTK_LIST_STORE(model),&iter,FRIEND_CHAT,create_unread_msg(),-1);
+				if (is_composing == TRUE)
+					gtk_list_store_set(GTK_LIST_STORE(model),&iter,FRIEND_CHAT,create_composing_unread_msg(),-1);
+				else
+					gtk_list_store_set(GTK_LIST_STORE(model),&iter,FRIEND_CHAT,create_unread_msg(),-1);
 			} else {
-				gtk_list_store_set(GTK_LIST_STORE(model),&iter,FRIEND_CHAT,create_chat_picture(),-1);
+				if (is_composing == TRUE)
+					gtk_list_store_set(GTK_LIST_STORE(model),&iter,FRIEND_CHAT,create_composing_chat_picture(),-1);
+				else
+					gtk_list_store_set(GTK_LIST_STORE(model),&iter,FRIEND_CHAT,create_chat_picture(),-1);
 			}
 		}while(gtk_tree_model_iter_next(model,&iter));
 	}
@@ -246,6 +266,20 @@ static gboolean grab_focus(GtkWidget *w){
 	return FALSE;
 }
 
+void linphone_gtk_friend_list_set_active_address(const LinphoneAddress *addr){
+	GtkWidget *w=linphone_gtk_get_main_window();
+	GtkWidget *friendlist=linphone_gtk_get_widget(w,"contact_list");
+	LinphoneAddress *old_addr=(LinphoneAddress*)g_object_get_data(G_OBJECT(friendlist),"from");
+	g_object_set_data(G_OBJECT(friendlist),"from", addr ? linphone_address_clone(addr) : NULL);
+	if (old_addr) linphone_address_unref(old_addr);
+}
+
+const LinphoneAddress *linphone_gtk_friend_list_get_active_address(void){
+	GtkWidget *w=linphone_gtk_get_main_window();
+	GtkWidget *friendlist=linphone_gtk_get_widget(w,"contact_list");
+	return (const LinphoneAddress*)g_object_get_data(G_OBJECT(friendlist),"from");
+}
+
 void linphone_gtk_friend_list_set_chat_conversation(const LinphoneAddress *la){
 	GtkTreeIter iter;
 	GtkListStore *store=NULL;
@@ -256,12 +290,11 @@ void linphone_gtk_friend_list_set_chat_conversation(const LinphoneAddress *la){
 	LinphoneFriend *lf=NULL;
 	LinphoneChatRoom *cr=NULL;
 	GtkNotebook *notebook=(GtkNotebook *)linphone_gtk_get_widget(w,"viewswitch");
-	char *la_str=linphone_address_as_string(la);
 	
-	lf=linphone_core_get_friend_by_address(linphone_gtk_get_core(),la_str);
+	lf=linphone_core_find_friend(linphone_gtk_get_core(),la);
 	if(lf==NULL){
 		cr=linphone_gtk_create_chatroom(la);
-		g_object_set_data(G_OBJECT(friendlist),"from",la_str);
+		linphone_gtk_friend_list_set_active_address(la);
 		if(chat_view==NULL){
 			chat_view=linphone_gtk_init_chatroom(cr,la);
 			g_object_set_data(G_OBJECT(friendlist),"chatview",(gpointer)chat_view);
@@ -276,17 +309,15 @@ void linphone_gtk_friend_list_set_chat_conversation(const LinphoneAddress *la){
 		if (gtk_tree_model_get_iter_first(model,&iter)) {
 			do{
 				const LinphoneAddress *uri;
-				char *lf_str;
 				gtk_tree_model_get(model, &iter,FRIEND_ID , &lf, -1);
 				uri=linphone_friend_get_address(lf);
-				lf_str=linphone_address_as_string(uri);
-				if( g_strcmp0(lf_str,la_str)==0){
+				if (linphone_address_weak_equal(uri,la)){
 					gtk_tree_model_get (model, &iter,FRIEND_CHATROOM , &cr, -1);
 					if(cr==NULL){
 						cr=linphone_gtk_create_chatroom(uri);
 						gtk_list_store_set(store,&iter,FRIEND_CHATROOM,cr,-1);
 					}
-					g_object_set_data(G_OBJECT(friendlist),"from",linphone_address_as_string(uri));
+					linphone_gtk_friend_list_set_active_address(uri);
 					if(chat_view==NULL){
 						chat_view=linphone_gtk_init_chatroom(cr,uri);
 						g_object_set_data(G_OBJECT(friendlist),"chatview",(gpointer)chat_view);
@@ -345,7 +376,7 @@ void linphone_gtk_chat_selected(GtkWidget *item){
 			gtk_list_store_set(store,&iter,FRIEND_CHATROOM,cr,-1);
 		}
 		page=(GtkWidget*)g_object_get_data(G_OBJECT(friendlist),"chatview");
-		g_object_set_data(G_OBJECT(friendlist),"from",linphone_address_as_string(uri));
+		linphone_gtk_friend_list_set_active_address(uri);
 		if(page==NULL){
 			page=linphone_gtk_init_chatroom(cr,uri);
 			g_object_set_data(G_OBJECT(friendlist),"chatview",(gpointer)page);
@@ -442,19 +473,23 @@ void linphone_gtk_my_presence_clicked(GtkWidget *button){
 static void icon_press_handler(GtkEntry *entry){
 	const char *text=gtk_entry_get_text(entry);
 	if (text && strlen(text)>0){
-		char *uri;
+		LinphoneAddress *addr;
 		LinphoneFriend *lf;
-		linphone_core_interpret_friend_uri(linphone_gtk_get_core(),text,&uri);
-		if (uri==NULL){
+		char *uri;
+		addr=linphone_core_interpret_url(linphone_gtk_get_core(),text);
+		if (addr==NULL){
 			return ;
 		}
+		uri=linphone_address_as_string_uri_only(addr);
 		lf=linphone_core_get_friend_by_address(linphone_gtk_get_core(),uri);
+		ms_free(uri);
 		if (lf==NULL)
-			lf=linphone_friend_new_with_addr(uri);
+			lf=linphone_friend_new();
 		if (lf!=NULL){
+			linphone_friend_set_address(lf,addr);
 			linphone_gtk_show_contact(lf);
 		}
-		ms_free(uri);
+		linphone_address_destroy(addr);
 	}
 }
 
@@ -476,11 +511,12 @@ static void check_contact(GtkEditable *editable, LinphoneCore *lc){
 	char *tmp=gtk_editable_get_chars(editable,0,-1);
 	if (tmp!=NULL){
 		if (strlen(tmp)>0){
-			char *uri=NULL;
-			linphone_core_interpret_friend_uri(lc,tmp,&uri);
-			if (uri){
+			LinphoneAddress *addr=linphone_core_interpret_url(lc,tmp);
+			if (addr){
+				char *uri=linphone_address_as_string_uri_only(addr);
 				LinphoneFriend *lf=linphone_core_get_friend_by_address(lc,uri);
 				ms_free(uri);
+				linphone_address_destroy(addr);
 				if (lf) {
 					update_star(GTK_ENTRY(editable),TRUE);
 					g_free(tmp);
@@ -862,7 +898,6 @@ void linphone_gtk_contact_ok(GtkWidget *button){
 	GtkWidget *w=gtk_widget_get_toplevel(button);
 	LinphoneFriend *lf=(LinphoneFriend*)g_object_get_data(G_OBJECT(w),"friend_ref");
 	LinphoneFriend *lf2;
-	char *fixed_uri=NULL;
 	gboolean show_presence=FALSE,allow_presence=FALSE;
 	const gchar *name,*uri;
 	LinphoneAddress* friend_address;
@@ -875,32 +910,36 @@ void linphone_gtk_contact_ok(GtkWidget *button){
 		linphone_friend_set_inc_subscribe_policy(lf,allow_presence ? LinphoneSPAccept : LinphoneSPDeny);
 		linphone_friend_send_subscribe(lf,show_presence);
 	}
-	name=gtk_entry_get_text(GTK_ENTRY(linphone_gtk_get_widget(w,"name")));
+
+	name = NULL;
+	if(gtk_entry_get_text_length(GTK_ENTRY(linphone_gtk_get_widget(w,"name"))) != 0){
+		name=gtk_entry_get_text(GTK_ENTRY(linphone_gtk_get_widget(w,"name")));
+	}
 	uri=gtk_entry_get_text(GTK_ENTRY(linphone_gtk_get_widget(w,"sip_address")));
 	show_presence=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(linphone_gtk_get_widget(w,"show_presence")));
 	allow_presence=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(linphone_gtk_get_widget(w,"allow_presence")));
-	linphone_core_interpret_friend_uri(linphone_gtk_get_core(),uri,&fixed_uri);
-	if (fixed_uri==NULL){
+	friend_address=linphone_core_interpret_url(linphone_gtk_get_core(),uri);
+	if (friend_address==NULL){
 		linphone_gtk_display_something(GTK_MESSAGE_WARNING,_("Invalid sip contact !"));
 		return ;
 	}
-	friend_address = linphone_address_new(fixed_uri);
-	linphone_address_set_display_name(friend_address,name);
-	linphone_friend_set_addr(lf,friend_address);
-	linphone_address_destroy(friend_address);
 
+	linphone_address_set_display_name(friend_address,name);
+	linphone_friend_set_name(lf,name);
+	linphone_friend_set_address(lf,friend_address);
 	linphone_friend_send_subscribe(lf,show_presence);
 	linphone_friend_set_inc_subscribe_policy(lf,allow_presence==TRUE ? LinphoneSPAccept : LinphoneSPDeny);
 	if (linphone_friend_in_list(lf)) {
 		linphone_friend_done(lf);
 	} else {
-		lf2=linphone_core_get_friend_by_address(linphone_gtk_get_core(),fixed_uri);
+		char *uri=linphone_address_as_string_uri_only(friend_address);
+		lf2=linphone_core_get_friend_by_address(linphone_gtk_get_core(),uri);
+		ms_free(uri);
 		if(lf2==NULL){
-			linphone_friend_set_name(lf,name);
 			linphone_core_add_friend(linphone_gtk_get_core(),lf);
 		}
 	}
-	ms_free(fixed_uri);
+	linphone_address_destroy(friend_address);
 	linphone_gtk_show_friends();
 	gtk_widget_destroy(w);
 }
@@ -1077,4 +1116,4 @@ gboolean linphone_gtk_contact_list_button_pressed(GtkWidget *widget, GdkEventBut
 void linphone_gtk_buddy_info_updated(LinphoneCore *lc, LinphoneFriend *lf){
 	/*refresh the entire list*/
 	linphone_gtk_show_friends();
-}
\ No newline at end of file
+}
