@@ -28,10 +28,12 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #endif
 #include "linphonecore.h"
 
+#include "ldap/ldapprovider.h"
+
 #ifdef ENABLE_NLS
 # include <libintl.h>
 # undef _
-# define _(String) gettext (String)
+# define _(String) dgettext (GETTEXT_PACKAGE,String)
 #else
 # define _(String) (String)
 # define ngettext(singular,plural,number) ((number>1) ? (plural) : (singular) )
@@ -46,6 +48,11 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #define LINPHONE_VERSION LINPHONE_VERSION_DATE
 #endif
 
+enum {
+	COMPLETION_HISTORY,
+	COMPLETION_LDAP
+};
+
 GdkPixbuf * create_pixbuf(const gchar *filename);
 GdkPixbufAnimation *create_pixbuf_animation(const gchar *filename);
 void add_pixmap_directory(const gchar *directory);
@@ -85,6 +92,11 @@ int linphone_gtk_get_ui_config_int(const char *key, int def);
 void linphone_gtk_set_ui_config_int(const char *key , int val);
 void linphone_gtk_visibility_set(const char *hiddens, const char *window_name, GtkWidget *w, gboolean show);
 
+LinphoneLDAPContactProvider* linphone_gtk_get_ldap(void);
+void linphone_gtk_set_ldap(LinphoneLDAPContactProvider* ldap);
+int linphone_gtk_is_ldap_supported(void);
+
+
 void linphone_gtk_open_browser(const char *url);
 void linphone_gtk_check_for_new_version(void);
 const char *linphone_gtk_get_lang(const char *config_file);
@@ -104,10 +116,13 @@ void linphone_gtk_send_text();
 GtkWidget * linphone_gtk_init_chatroom(LinphoneChatRoom *cr, const LinphoneAddress *with);
 LinphoneChatRoom * linphone_gtk_create_chatroom(const LinphoneAddress *with);
 void linphone_gtk_text_received(LinphoneCore *lc, LinphoneChatRoom *room, LinphoneChatMessage *msg);
+void linphone_gtk_is_composing_received(LinphoneCore *lc, LinphoneChatRoom *room);
 
 void linphone_gtk_friend_list_update_chat_picture();
 void linphone_gtk_friend_list_set_chat_conversation(const LinphoneAddress *la);
 gboolean linphone_gtk_friend_list_is_contact(const LinphoneAddress *addr);
+void linphone_gtk_friend_list_set_active_address(const LinphoneAddress *addr);
+const LinphoneAddress *linphone_gtk_friend_list_get_active_address(void);
 void linphone_gtk_notebook_tab_select(GtkNotebook *notebook,GtkWidget *page,guint page_num, gpointer data);
 void linphone_gtk_show_friends(void);
 void linphone_gtk_show_contact(LinphoneFriend *lf);
@@ -138,7 +153,7 @@ void linphone_gtk_update_video_button(LinphoneCall *call);
 typedef float (*get_volume_t)(void *data);
 void linphone_gtk_init_audio_meter(GtkWidget *w, get_volume_t get_volume, void *data);
 
-void linphone_gtk_show_login_frame(LinphoneProxyConfig *cfg);
+void linphone_gtk_show_login_frame(LinphoneProxyConfig *cfg, gboolean disable_auto_login);
 void linphone_gtk_exit_login_frame(void);
 void linphone_gtk_set_ui_config(const char *key, const char *value);
 
@@ -149,4 +164,10 @@ void linphone_gtk_uninit_instance(void);
 void linphone_gtk_monitor_usb(void);
 void linphone_gtk_unmonitor_usb(void);
 
-gchar *linphone_gtk_get_record_path(const LinphoneAddress *address, gboolean is_conference);
\ No newline at end of file
+gchar *linphone_gtk_get_record_path(const LinphoneAddress *address, gboolean is_conference);
+void linphone_gtk_schedule_restart(void);
+
+void linphone_gtk_set_configuration_uri(void);
+GtkWidget * linphone_gtk_show_config_fetching(void);
+void linphone_gtk_close_config_fetching(GtkWidget *w, LinphoneConfiguringState state);
+
