@@ -20,15 +20,12 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "linphone.h"
 
 void linphone_gtk_login_frame_connect_clicked(GtkWidget *button);
+void test_button_clicked_cb(GtkWidget *button);
 void linphone_gtk_exit_login_frame(void);
 
-enum {
-	NetworkKindAdsl,
-	NetworkKindOpticalFiber
-};
 
-static void do_login(SipSetupContext *ssctx, const char *identity, const char * passwd){
-	if (sip_setup_context_login_account(ssctx,identity,passwd)==0){
+static void do_login(SipSetupContext *ssctx, const char *identity, const char * passwd, const char *userid){
+	if (sip_setup_context_login_account(ssctx,identity,passwd,userid)==0){
 	}
 }
 
@@ -40,44 +37,38 @@ static gboolean do_login_noprompt(LinphoneProxyConfig *cfg){
 	if (ssctx==NULL) return TRUE;/*not ready ?*/
 	username=linphone_gtk_get_ui_config ("login_username",NULL);
 	if (username==NULL) {
-		linphone_gtk_set_ui_config_int("automatic_login",0);
-		linphone_gtk_show_login_frame(cfg);
+		linphone_gtk_show_login_frame(cfg,TRUE);
 		return FALSE;
 	}
 	addr=linphone_address_new(linphone_proxy_config_get_identity(cfg));
 	linphone_address_set_username(addr,username);
 	tmp=linphone_address_as_string (addr);
-	do_login(ssctx,tmp,NULL);
+	do_login(ssctx,tmp,NULL,NULL);
 	linphone_address_destroy(addr);
 	linphone_gtk_load_identities();
 	return FALSE;
 }
 
-void linphone_gtk_show_login_frame(LinphoneProxyConfig *cfg){
+void linphone_gtk_show_login_frame(LinphoneProxyConfig *cfg, gboolean disable_auto_login){
 	GtkWidget *mw=linphone_gtk_get_main_window();
 	GtkWidget *label=linphone_gtk_get_widget(mw,"login_label");
 	const LinphoneAuthInfo *ai;
 	gchar *str;
 	LinphoneAddress *from;
 	LinphoneCore *lc=linphone_gtk_get_core();
-	int nettype;
 	const char *passwd=NULL;
-
+	const char *userid=NULL;
+	gboolean auto_login=linphone_gtk_get_ui_config_int("automatic_login",0);
 	
-	if (linphone_core_get_download_bandwidth(lc)==512 &&
-		linphone_core_get_upload_bandwidth(lc)==512)
-		nettype=NetworkKindOpticalFiber;
-	else nettype=NetworkKindAdsl;
-	gtk_combo_box_set_active(GTK_COMBO_BOX(linphone_gtk_get_widget(mw,"login_internet_kind")),nettype);
-	//gtk_combo_box_set_active(GTK_COMBO_BOX(linphone_gtk_get_widget(mw,"internet_kind")),nettype);
-	
-	if (linphone_gtk_get_ui_config_int("automatic_login",0) ){
+	if (auto_login && !disable_auto_login){
 		g_timeout_add(250,(GSourceFunc)do_login_noprompt,cfg);
 		return;
 	}
 
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(linphone_gtk_get_widget(mw,"automatic_login")),auto_login);
+	
 	{
-		const char *login_image=linphone_gtk_get_ui_config("login_image",NULL);
+		const char *login_image=linphone_gtk_get_ui_config("login_image","linphone-banner.png");
 		if (login_image){
 			GdkPixbuf *pbuf=create_pixbuf (login_image);
 			gtk_image_set_from_pixbuf (GTK_IMAGE(linphone_gtk_get_widget(mw,"login_image")),
@@ -89,6 +80,10 @@ void linphone_gtk_show_login_frame(LinphoneProxyConfig *cfg){
 	gtk_widget_hide(linphone_gtk_get_widget(mw,"disconnect_item"));
 	gtk_widget_hide(linphone_gtk_get_widget(mw,"main_frame"));
 	gtk_widget_show(linphone_gtk_get_widget(mw,"login_frame"));
+	if (linphone_gtk_get_ui_config_int("login_needs_userid",FALSE)){
+		gtk_widget_show(linphone_gtk_get_widget(mw,"userid"));
+		gtk_widget_show(linphone_gtk_get_widget(mw,"login_userid"));
+	}
 	gtk_widget_set_sensitive(linphone_gtk_get_widget(mw,"options_menu"),FALSE);
 	str=g_strdup_printf(_("Please enter login information for %s"),linphone_proxy_config_get_domain(cfg));
 	gtk_label_set_text(GTK_LABEL(label),str);
@@ -102,14 +97,19 @@ void linphone_gtk_show_login_frame(LinphoneProxyConfig *cfg){
 			linphone_address_set_username(from,username);
 	}
 	
-	ai=linphone_core_find_auth_info(lc,linphone_proxy_config_get_domain(cfg),linphone_address_get_username(from));
+	ai=linphone_core_find_auth_info(lc,linphone_proxy_config_get_domain(cfg),linphone_address_get_username(from),NULL);
 	/*display the last entered username, if not '?????'*/
 	if (linphone_address_get_username(from)[0]!='?')
 		gtk_entry_set_text(GTK_ENTRY(linphone_gtk_get_widget(mw,"login_username")),
 			linphone_address_get_username(from));
-	if (ai) passwd=linphone_auth_info_get_passwd(ai);
+	if (ai) {
+		passwd=linphone_auth_info_get_passwd(ai);
+		userid=linphone_auth_info_get_userid(ai);
+	}
 	gtk_entry_set_text(GTK_ENTRY(linphone_gtk_get_widget(mw,"login_password")),
 		passwd!=NULL ? passwd : "");
+	gtk_entry_set_text(GTK_ENTRY(linphone_gtk_get_widget(mw,"login_userid")),
+		userid ? userid : "");
 	
 	linphone_address_destroy(from);
 }
@@ -130,8 +130,7 @@ void linphone_gtk_logout_clicked(){
 		SipSetupContext *ss=linphone_proxy_config_get_sip_setup_context(cfg);
 		if (ss){
 			sip_setup_context_logout(ss);
-			linphone_gtk_set_ui_config_int("automatic_login",FALSE);
-			linphone_gtk_show_login_frame(cfg);
+			linphone_gtk_show_login_frame(cfg,TRUE);
 		}
 	}
 }
@@ -142,6 +141,7 @@ void linphone_gtk_login_frame_connect_clicked(GtkWidget *button){
 	GtkWidget *mw=gtk_widget_get_toplevel(button);
 	const char *username;
 	const char *password;
+	const char *userid;
 	char *identity;
 	gboolean autologin;
 	LinphoneProxyConfig *cfg=(LinphoneProxyConfig*)g_object_get_data(G_OBJECT(mw),"login_proxy_config");
@@ -150,7 +150,8 @@ void linphone_gtk_login_frame_connect_clicked(GtkWidget *button){
 
 	username=gtk_entry_get_text(GTK_ENTRY(linphone_gtk_get_widget(mw,"login_username")));
 	password=gtk_entry_get_text(GTK_ENTRY(linphone_gtk_get_widget(mw,"login_password")));
-
+	userid=gtk_entry_get_text(GTK_ENTRY(linphone_gtk_get_widget(mw,"login_userid")));
+	
 	if (username==NULL || username[0]=='\0')
 		return;
 
@@ -161,19 +162,7 @@ void linphone_gtk_login_frame_connect_clicked(GtkWidget *button){
 	from=linphone_address_new(linphone_proxy_config_get_identity(cfg));
 	linphone_address_set_username(from,username);
 	identity=linphone_address_as_string(from);
-	do_login(ssctx,identity,password);
+	do_login(ssctx,identity,password,userid);
 	/*we need to refresh the identities since the proxy config may have changed.*/
 	linphone_gtk_load_identities();
 }
-
-void linphone_gtk_internet_kind_changed(GtkWidget *combo){
-	int netkind_id=gtk_combo_box_get_active(GTK_COMBO_BOX(combo));
-	LinphoneCore *lc=linphone_gtk_get_core();
-	if (netkind_id==NetworkKindAdsl){
-		linphone_core_set_upload_bandwidth(lc,256);
-		linphone_core_set_download_bandwidth(lc,512);
-	}else if (netkind_id==NetworkKindOpticalFiber){
-		linphone_core_set_upload_bandwidth(lc,512);
-		linphone_core_set_download_bandwidth(lc,512);
-	}
-}
