@@ -34,6 +34,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 #ifdef WIN32
 #define chdir _chdir
+#include "direct.h"
 #endif
 
 #if defined(HAVE_NOTIFY1) || defined(HAVE_NOTIFY4)
@@ -50,12 +51,15 @@ const char *this_program_ident_string="linphone_ident_string=" LINPHONE_VERSION;
 
 static LinphoneCore *the_core=NULL;
 static GtkWidget *the_ui=NULL;
+static LinphoneLDAPContactProvider* ldap_provider = NULL;
 
+static void linphone_gtk_global_state_changed(LinphoneCore *lc, LinphoneGlobalState state, const char*str);
 static void linphone_gtk_registration_state_changed(LinphoneCore *lc, LinphoneProxyConfig *cfg, LinphoneRegistrationState rs, const char *msg);
 static void linphone_gtk_notify_recv(LinphoneCore *lc, LinphoneFriend * fid);
 static void linphone_gtk_new_unknown_subscriber(LinphoneCore *lc, LinphoneFriend *lf, const char *url);
-static void linphone_gtk_auth_info_requested(LinphoneCore *lc, const char *realm, const char *username);
+static void linphone_gtk_auth_info_requested(LinphoneCore *lc, const char *realm, const char *username, const char *domain);
 static void linphone_gtk_display_status(LinphoneCore *lc, const char *status);
+static void linphone_gtk_configuring_status(LinphoneCore *lc, LinphoneConfiguringState status, const char *message);
 static void linphone_gtk_display_message(LinphoneCore *lc, const char *msg);
 static void linphone_gtk_display_warning(LinphoneCore *lc, const char *warning);
 static void linphone_gtk_display_url(LinphoneCore *lc, const char *msg, const char *url);
@@ -63,17 +67,20 @@ static void linphone_gtk_call_log_updated(LinphoneCore *lc, LinphoneCallLog *cl)
 static void linphone_gtk_call_state_changed(LinphoneCore *lc, LinphoneCall *call, LinphoneCallState cs, const char *msg);
 static void linphone_gtk_call_encryption_changed(LinphoneCore *lc, LinphoneCall *call, bool_t enabled, const char *token);
 static void linphone_gtk_transfer_state_changed(LinphoneCore *lc, LinphoneCall *call, LinphoneCallState cstate);
+static void linphone_gtk_dtmf_received(LinphoneCore *lc, LinphoneCall *call, int dtmf);
 void linphone_gtk_save_main_window_position(GtkWindow* mw, GdkEvent *event, gpointer data);
 static gboolean linphone_gtk_auto_answer(LinphoneCall *call);
 void linphone_gtk_status_icon_set_blinking(gboolean val);
 void _linphone_gtk_enable_video(gboolean val);
-
+void linphone_gtk_on_uribar_changed(GtkEditable *uribar, gpointer user_data);
+static void linphone_gtk_init_ui(void);
 
 #ifndef HAVE_GTK_OSX
 static gint main_window_x=0;
 static gint main_window_y=0;
 #endif
 static gboolean verbose=0;
+static gboolean quit_done=FALSE;
 static gboolean auto_answer = 0;
 static gchar * addr_to_call = NULL;
 static gboolean no_video=FALSE;
@@ -82,6 +89,9 @@ static gchar *workingdir=NULL;
 static char *progpath=NULL;
 gchar *linphone_logfile=NULL;
 static gboolean workaround_gtk_entry_chinese_bug=FALSE;
+static gchar *custom_config_file=NULL;
+static gboolean restart=FALSE;
+static GtkWidget *config_fetching_dialog=NULL;
 
 static GOptionEntry linphone_options[]={
 	{
@@ -133,6 +143,13 @@ static GOptionEntry linphone_options[]={
 	    .arg_data = (gpointer) & workingdir,
 	    .description = N_("Specifiy a working directory (should be the base of the installation, eg: c:\\Program Files\\Linphone)")
 	},
+	{
+		.long_name = "config",
+		.short_name = '\0',
+		.arg = G_OPTION_ARG_FILENAME,
+		.arg_data = (gpointer) &custom_config_file,
+		.description = N_("Configuration file")
+	},
 	{0}
 };
 
@@ -155,7 +172,9 @@ char *linphone_gtk_get_config_file(const char *filename){
 	/*try accessing a local file first if exists*/
 	if (access(CONFIG_FILE,F_OK)==0){
 		snprintf(config_file,path_max,"%s",filename);
-	}else{
+	} else if (g_path_is_absolute(filename)) {
+		snprintf(config_file,path_max,"%s",filename);
+	} else{
 #ifdef WIN32
 		const char *appdata=getenv("APPDATA");
 		if (appdata){
@@ -219,15 +238,37 @@ static const char *linphone_gtk_get_factory_config_file(){
 	return _factory_config_file;
 }
 
+LinphoneLDAPContactProvider* linphone_gtk_get_ldap(void){
+	return ldap_provider;
+}
+
+int linphone_gtk_is_ldap_supported(void){
+	return linphone_ldap_contact_provider_available();
+}
+
+void linphone_gtk_set_ldap(LinphoneLDAPContactProvider* ldap)
+{
+	if( ldap_provider )
+		linphone_contact_provider_unref(ldap_provider);
+
+	ldap_provider = ldap ? linphone_ldap_contact_provider_ref( ldap )
+						 : NULL;
+}
+
+void linphone_gtk_schedule_restart(void){
+	restart=TRUE;
+}
+
 static void linphone_gtk_init_liblinphone(const char *config_file,
 		const char *factory_config_file, const char *db_file) {
 	LinphoneCoreVTable vtable={0};
 	gchar *secrets_file=linphone_gtk_get_config_file(SECRETS_FILE);
 
+	vtable.global_state_changed=linphone_gtk_global_state_changed;
 	vtable.call_state_changed=linphone_gtk_call_state_changed;
 	vtable.registration_state_changed=linphone_gtk_registration_state_changed;
-	vtable.notify_presence_recv=linphone_gtk_notify_recv;
-	vtable.new_subscription_request=linphone_gtk_new_unknown_subscriber;
+	vtable.notify_presence_received=linphone_gtk_notify_recv;
+	vtable.new_subscription_requested=linphone_gtk_new_unknown_subscriber;
 	vtable.auth_info_requested=linphone_gtk_auth_info_requested;
 	vtable.display_status=linphone_gtk_display_status;
 	vtable.display_message=linphone_gtk_display_message;
@@ -236,18 +277,31 @@ static void linphone_gtk_init_liblinphone(const char *config_file,
 	vtable.call_log_updated=linphone_gtk_call_log_updated;
 	//vtable.text_received=linphone_gtk_text_received;
 	vtable.message_received=linphone_gtk_text_received;
+	vtable.is_composing_received=linphone_gtk_is_composing_received;
 	vtable.refer_received=linphone_gtk_refer_received;
 	vtable.buddy_info_updated=linphone_gtk_buddy_info_updated;
 	vtable.call_encryption_changed=linphone_gtk_call_encryption_changed;
 	vtable.transfer_state_changed=linphone_gtk_transfer_state_changed;
+	vtable.dtmf_received=linphone_gtk_dtmf_received;
+	vtable.configuring_status=linphone_gtk_configuring_status;
 
 	the_core=linphone_core_new(&vtable,config_file,factory_config_file,NULL);
+	linphone_core_migrate_to_multi_transport(the_core);
 	//lp_config_set_int(linphone_core_get_config(the_core), "sip", "store_auth_info", 0);
+
+
+	if( lp_config_has_section(linphone_core_get_config(the_core),"ldap") ){
+		LpConfig* cfg = linphone_core_get_config(the_core);
+		LinphoneDictionary* ldap_cfg = lp_config_section_to_dict(cfg, "ldap");
+		linphone_gtk_set_ldap( linphone_ldap_contact_provider_create(the_core, ldap_cfg) );
+	}
+
 	linphone_core_set_user_agent(the_core,"Linphone", LINPHONE_VERSION);
 	linphone_core_set_waiting_callback(the_core,linphone_gtk_wait,NULL);
 	linphone_core_set_zrtp_secrets_file(the_core,secrets_file);
 	g_free(secrets_file);
-	linphone_core_enable_video(the_core,TRUE,TRUE);
+	linphone_core_enable_video_capture(the_core, TRUE);
+	linphone_core_enable_video_display(the_core, TRUE);
 	if (no_video) {
 		_linphone_gtk_enable_video(FALSE);
 		linphone_gtk_set_ui_config_int("videoselfview",0);
@@ -264,7 +318,7 @@ GtkWidget *linphone_gtk_get_main_window(){
 }
 
 void linphone_gtk_destroy_main_window() {
-	linphone_gtk_destroy_window(the_ui);	
+	linphone_gtk_destroy_window(the_ui);
 	the_ui = NULL;
 }
 
@@ -320,6 +374,8 @@ GtkWidget *linphone_gtk_create_window(const char *window_name){
 
 	if (get_ui_file(window_name,path,sizeof(path))==-1) return NULL;
 
+	gtk_builder_set_translation_domain(builder,GETTEXT_PACKAGE);
+
 	if (!gtk_builder_add_from_file (builder, path, &error)){
 		g_error("Couldn't load builder file: %s", error->message);
 		g_error_free (error);
@@ -346,6 +402,9 @@ GtkWidget *linphone_gtk_create_widget(const char *filename, const char *widget_n
 	object_ids[1]=NULL;
 
 	if (get_ui_file(filename,path,sizeof(path))==-1) return NULL;
+	
+	gtk_builder_set_translation_domain(builder,GETTEXT_PACKAGE);
+	
 	if (!gtk_builder_add_objects_from_file(builder,path,object_ids,&error)){
 		g_error("Couldn't load %s from builder file %s: %s", widget_name,path,error->message);
 		g_error_free (error);
@@ -614,12 +673,31 @@ static gboolean linphone_gtk_iterate(LinphoneCore *lc){
 	return TRUE;
 }
 
+static gboolean uribar_completion_matchfunc(GtkEntryCompletion *completion, const gchar *key, GtkTreeIter *iter, gpointer user_data){
+	char* address = NULL;
+	gboolean ret  = FALSE;
+	gchar *tmp= NULL;
+	gtk_tree_model_get(gtk_entry_completion_get_model(completion),iter,0,&address,-1);
+
+	tmp = g_utf8_casefold(address,-1);
+	if (tmp){
+		if (strstr(tmp,key))
+			ret=TRUE;
+		g_free(tmp);
+	}
+
+	if( address)
+		g_free(address);
+
+	return ret;
+}
+
 static void load_uri_history(){
 	GtkEntry *uribar=GTK_ENTRY(linphone_gtk_get_widget(linphone_gtk_get_main_window(),"uribar"));
 	char key[20];
 	int i;
 	GtkEntryCompletion *gep=gtk_entry_completion_new();
-	GtkListStore *model=gtk_list_store_new(1,G_TYPE_STRING);
+	GtkListStore *model=gtk_list_store_new(2,G_TYPE_STRING,G_TYPE_INT);
 	for (i=0;;i++){
 		const char *uri;
 		snprintf(key,sizeof(key),"uri%i",i);
@@ -627,14 +705,18 @@ static void load_uri_history(){
 		if (uri!=NULL) {
 			GtkTreeIter iter;
 			gtk_list_store_append(model,&iter);
-			gtk_list_store_set(model,&iter,0,uri,-1);
+			gtk_list_store_set(model,&iter,0,uri,1,COMPLETION_HISTORY,-1);
 			if (i==0) gtk_entry_set_text(uribar,uri);
 		}
 		else break;
 	}
 	gtk_entry_completion_set_model(gep,GTK_TREE_MODEL(model));
 	gtk_entry_completion_set_text_column(gep,0);
+	gtk_entry_completion_set_popup_completion(gep, TRUE);
+	gtk_entry_completion_set_match_func(gep,uribar_completion_matchfunc, NULL, NULL);
+	gtk_entry_completion_set_minimum_key_length(gep,3);
 	gtk_entry_set_completion(uribar,gep);
+	g_signal_connect (G_OBJECT (uribar), "changed", G_CALLBACK(linphone_gtk_on_uribar_changed), NULL);
 }
 
 static void save_uri_history(){
@@ -682,10 +764,129 @@ static void completion_add_text(GtkEntry *entry, const char *text){
 	}
 	/* and prepend it on top of the list */
 	gtk_list_store_prepend(GTK_LIST_STORE(model),&iter);
-	gtk_list_store_set(GTK_LIST_STORE(model),&iter,0,text,-1);
+	gtk_list_store_set(GTK_LIST_STORE(model),&iter,0,text,1,COMPLETION_HISTORY,-1);
 	save_uri_history();
 }
 
+void on_contact_provider_search_results( LinphoneContactSearch* req, MSList* friends, void* data )
+{
+	GtkTreeIter    iter;
+	GtkEntry*    uribar = GTK_ENTRY(data);
+	GtkEntryCompletion* compl = gtk_entry_get_completion(uribar);
+	GtkTreeModel* model = gtk_entry_completion_get_model(compl);
+	GtkListStore*  list = GTK_LIST_STORE(model);
+	LinphoneLDAPContactSearch* search = linphone_ldap_contact_search_cast(req);
+	gboolean valid;
+
+	// clear completion list from previous non-history entries
+	valid = gtk_tree_model_get_iter_first(model,&iter);
+	while(valid)
+	{
+		char* url;
+		int type;
+		gtk_tree_model_get(model,&iter, 0,&url, 1,&type, -1);
+
+		if (type != COMPLETION_HISTORY) {
+			valid = gtk_list_store_remove(list, &iter);
+		} else {
+			valid = gtk_tree_model_iter_next(model,&iter);
+		}
+
+		if( url ) g_free(url);
+		if( !valid ) break;
+	}
+
+	// add new non-history related matches
+	while( friends ){
+		LinphoneFriend* lf = friends->data;
+		if( lf ) {
+			const LinphoneAddress* la = linphone_friend_get_address(lf);
+			if( la ){
+				char *addr = linphone_address_as_string(la);
+
+				if( addr ){
+					ms_message("[LDAP]Insert match: %s",  addr);
+					gtk_list_store_insert_with_values(list, &iter, -1,
+													  0, addr,
+													  1, COMPLETION_LDAP, -1);
+					ms_free(addr);
+				}
+			}
+		}
+		friends = friends->next;
+	}
+	gtk_entry_completion_complete(compl);
+	// save the number of LDAP results to better decide if new results should be fetched when search predicate gets bigger
+	gtk_object_set_data(GTK_OBJECT(uribar), "ldap_res_cout",
+						GINT_TO_POINTER(
+							linphone_ldap_contact_search_result_count(search)
+							)
+						);
+
+	// Gtk bug? we need to emit a "changed" signal so that the completion appears if
+	// the list of results was previously empty
+	g_signal_handlers_block_by_func(uribar, linphone_gtk_on_uribar_changed, NULL);
+	g_signal_emit_by_name(uribar, "changed");
+	g_signal_handlers_unblock_by_func(uribar, linphone_gtk_on_uribar_changed, NULL);
+}
+
+struct CompletionTimeout {
+	guint timeout_id;
+};
+
+static gboolean launch_contact_provider_search(void *userdata)
+{
+	LinphoneLDAPContactProvider* ldap = linphone_gtk_get_ldap();
+	GtkWidget*      uribar = GTK_WIDGET(userdata);
+	const gchar* predicate = gtk_entry_get_text(GTK_ENTRY(uribar));
+	gchar* previous_search = gtk_object_get_data(GTK_OBJECT(uribar), "previous_search");
+	unsigned int prev_res_count = GPOINTER_TO_INT(gtk_object_get_data(GTK_OBJECT(uribar), "ldap_res_cout"));
+
+	if( ldap && strlen(predicate) >= 3 ){ // don't search too small predicates
+		unsigned int max_res_count = linphone_ldap_contact_provider_get_max_result(ldap);
+
+		if( previous_search  &&
+			(strstr(predicate, previous_search) == predicate) && // last search contained results from this one
+			(prev_res_count != max_res_count) ){ // and we didn't reach the max result limit
+
+			ms_message("Don't launch search on already searched data (current: %s, old search: %s), (%d/%d results)",
+					   predicate, previous_search, prev_res_count, max_res_count);
+			return FALSE;
+		}
+
+		// save current search
+		if( previous_search ) ms_free(previous_search);
+		gtk_object_set_data(GTK_OBJECT(uribar), "previous_search", ms_strdup(predicate));
+
+		ms_message("launch_contact_provider_search");
+		LinphoneContactSearch* search =linphone_contact_provider_begin_search(
+					linphone_contact_provider_cast(ldap_provider),
+					predicate, on_contact_provider_search_results, uribar
+					);
+
+		if(search)
+			linphone_contact_search_ref(search);
+	}
+	return FALSE;
+}
+
+void linphone_gtk_on_uribar_changed(GtkEditable *uribar, gpointer user_data)
+{
+	if( linphone_gtk_get_ldap() ) {
+		gchar* text = gtk_editable_get_chars(uribar, 0,-1);
+		gint timeout = GPOINTER_TO_INT(gtk_object_get_data(GTK_OBJECT(uribar), "complete_timeout"));
+		if( text ) g_free(text);
+
+		if( timeout != 0 ) {
+			g_source_remove(timeout);
+		}
+
+		timeout = g_timeout_add_seconds(1,(GSourceFunc)launch_contact_provider_search, uribar);
+
+		gtk_object_set_data(GTK_OBJECT(uribar),"complete_timeout", GINT_TO_POINTER(timeout) );
+	}
+}
+
 bool_t linphone_gtk_video_enabled(void){
 	const LinphoneVideoPolicy *vpol=linphone_core_get_video_policy(linphone_gtk_get_core());
 	return vpol->automatically_accept && vpol->automatically_initiate;
@@ -784,6 +985,9 @@ gchar *linphone_gtk_get_record_path(const LinphoneAddress *address, gboolean is_
 			date,
 			id);
 	}
+	if (!dir) {
+		ms_message ("No directory for music, using [%s] instead",dir=getenv("HOME"));
+	}
 	return g_build_filename(dir,filename,NULL);
 }
 
@@ -874,7 +1078,8 @@ void linphone_gtk_answer_clicked(GtkWidget *button){
 void _linphone_gtk_enable_video(gboolean val){
 	LinphoneVideoPolicy policy={0};
 	policy.automatically_initiate=policy.automatically_accept=val;
-	linphone_core_enable_video(linphone_gtk_get_core(),TRUE,TRUE);
+	linphone_core_enable_video_capture(linphone_gtk_get_core(), TRUE);
+	linphone_core_enable_video_display(linphone_gtk_get_core(), TRUE);
 	linphone_core_set_video_policy(linphone_gtk_get_core(),&policy);
 
 	if (val){
@@ -1007,7 +1212,7 @@ void linphone_gtk_password_ok(GtkWidget *w){
 	gtk_widget_destroy(window);
 }
 
-static void linphone_gtk_auth_info_requested(LinphoneCore *lc, const char *realm, const char *username){
+static void linphone_gtk_auth_info_requested(LinphoneCore *lc, const char *realm, const char *username, const char *domain){
 	GtkWidget *w=linphone_gtk_create_window("password");
 	GtkWidget *label=linphone_gtk_get_widget(w,"message");
 	LinphoneAuthInfo *info;
@@ -1020,18 +1225,22 @@ static void linphone_gtk_auth_info_requested(LinphoneCore *lc, const char *realm
 		return;
 	}
 
-	msg=g_strdup_printf(_("Please enter your password for username <i>%s</i>\n at domain <i>%s</i>:"),
+	msg=g_strdup_printf(_("Please enter your password for username <i>%s</i>\n at realm <i>%s</i>:"),
 		username,realm);
 	gtk_label_set_markup(GTK_LABEL(label),msg);
 	g_free(msg);
 	gtk_entry_set_text(GTK_ENTRY(linphone_gtk_get_widget(w,"userid_entry")),username);
-	info=linphone_auth_info_new(username, NULL, NULL, NULL,realm);
+	info=linphone_auth_info_new(username, NULL, NULL, NULL,realm,domain);
 	g_object_set_data(G_OBJECT(w),"auth_info",info);
 	g_object_weak_ref(G_OBJECT(w),(GWeakNotify)linphone_auth_info_destroy,info);
 	gtk_widget_show(w);
 	auth_timeout_new(w);
 }
 
+static void linphone_gtk_dtmf_received(LinphoneCore *lc, LinphoneCall *call, int dtmf){
+	ms_message("Dtmf %c received.",dtmf);
+}
+
 static void linphone_gtk_display_status(LinphoneCore *lc, const char *status){
 	GtkWidget *w=linphone_gtk_get_main_window();
 	GtkWidget *status_bar=linphone_gtk_get_widget(w,"status_bar");
@@ -1041,6 +1250,11 @@ static void linphone_gtk_display_status(LinphoneCore *lc, const char *status){
 			status);
 }
 
+static void linphone_gtk_configuring_status(LinphoneCore *lc, LinphoneConfiguringState status, const char *message) {
+	if (config_fetching_dialog) linphone_gtk_close_config_fetching(config_fetching_dialog, status);
+	config_fetching_dialog=NULL;
+}
+
 static void linphone_gtk_display_message(LinphoneCore *lc, const char *msg){
 	linphone_gtk_display_something(GTK_MESSAGE_INFO,msg);
 }
@@ -1150,6 +1364,24 @@ void linphone_gtk_notify(LinphoneCall *call, const char *msg){
 	}
 }
 
+static void linphone_gtk_global_state_changed(LinphoneCore *lc, LinphoneGlobalState state, const char*str){
+	switch(state){
+		case LinphoneGlobalStartup:
+			the_core=lc;
+		break;
+		case LinphoneGlobalConfiguring:
+			if (linphone_core_get_provisioning_uri(lc)){
+				config_fetching_dialog=linphone_gtk_show_config_fetching();
+			}
+		break;
+		case LinphoneGlobalOn:
+			linphone_gtk_init_ui();
+		break;
+		default:
+		break;
+	}
+}
+
 static void on_call_updated_response(GtkWidget *dialog, gint responseid, LinphoneCall *call){
 	if (linphone_call_get_state(call)==LinphoneCallUpdatedByRemote){
 		LinphoneCore *lc=linphone_call_get_core(call);
@@ -1401,7 +1633,7 @@ static void linphone_gtk_init_status_icon(){
 	const char *title;
 	title=linphone_gtk_get_ui_config("title",_("Linphone - a video internet phone"));
 	icon=gtk_status_icon_new_from_pixbuf(pbuf);
-#if GTK_CHECK_VERSION(2,20,0)
+#if GTK_CHECK_VERSION(2,20,2)
 	gtk_status_icon_set_name(icon,title);
 #endif
 	g_signal_connect_swapped(G_OBJECT(icon),"activate",(GCallback)handle_icon_click,NULL);
@@ -1643,7 +1875,7 @@ void linphone_gtk_manage_login(void){
 	if (cfg){
 		SipSetup *ss=linphone_proxy_config_get_sip_setup(cfg);
 		if (ss && (sip_setup_get_capabilities(ss) & SIP_SETUP_CAP_LOGIN)){
-			linphone_gtk_show_login_frame(cfg);
+			linphone_gtk_show_login_frame(cfg,FALSE);
 		}
 	}
 }
@@ -1698,6 +1930,40 @@ void linphone_gtk_init_dtmf_table(GtkWidget *mw){
 	g_object_set_data(G_OBJECT(linphone_gtk_get_widget(mw,"dtmf_*")),"label","*");
 }
 
+static gboolean key_allowed(guint32 code){
+	static const char *allowed="1234567890#*ABCD";
+	return code!=0 && strchr(allowed,(char)code)!=NULL;
+}
+
+static GtkButton *get_button_from_key(GtkWidget *w, GdkEvent *event){
+	guint keyval=event->key.keyval;
+	guint32 code=gdk_keyval_to_unicode(keyval);
+	code=g_unichar_toupper(code);
+	if (key_allowed(code)){
+		char widgetname[16];
+		w=gtk_widget_get_toplevel(w);
+		snprintf(widgetname,sizeof(widgetname),"dtmf_%c",code);
+		return GTK_BUTTON(linphone_gtk_get_widget(w,widgetname));
+	}
+	return NULL;
+}
+
+void linphone_gtk_keypad_key_pressed(GtkWidget *w, GdkEvent *event, gpointer userdata){
+	GtkButton *button=get_button_from_key(w,event);
+	if (button) {
+		linphone_gtk_dtmf_pressed(button);
+		/*g_signal_emit_by_name(button, "button-press-event");*/
+	}
+}
+
+void linphone_gtk_keypad_key_released(GtkWidget *w, GdkEvent *event, gpointer userdata){
+	GtkButton *button=get_button_from_key(w,event);
+	if (button) {
+		linphone_gtk_dtmf_released(button);
+		/*g_signal_emit_by_name(button, "button-release-event");*/
+	}
+}
+
 void linphone_gtk_create_keypad(GtkWidget *button){
 	GtkWidget *mw=linphone_gtk_get_main_window();
 	GtkWidget *k=(GtkWidget *)g_object_get_data(G_OBJECT(mw),"keypad");
@@ -1808,22 +2074,32 @@ static void linphone_gtk_check_soundcards(){
 	}
 }
 
+static void linphone_gtk_quit_core(void){
+	linphone_gtk_unmonitor_usb();
+	g_source_remove_by_user_data(linphone_gtk_get_core());
+#ifdef BUILD_WIZARD
+	linphone_gtk_close_assistant();
+#endif
+	linphone_gtk_set_ldap(NULL);
+	linphone_gtk_destroy_log_window();
+	linphone_core_destroy(the_core);
+	linphone_gtk_log_uninit();
+}
+
 static void linphone_gtk_quit(void){
-	static gboolean quit_done=FALSE;
 	if (!quit_done){
 		quit_done=TRUE;
-		linphone_gtk_unmonitor_usb();
-		g_source_remove_by_user_data(linphone_gtk_get_core());
-#ifdef BUILD_WIZARD
-		linphone_gtk_close_assistant();
-#endif
+		linphone_gtk_quit_core();
 		linphone_gtk_uninit_instance();
-		linphone_gtk_destroy_log_window();
-		linphone_core_destroy(the_core);
-		linphone_gtk_log_uninit();
+#ifndef HAVE_GTK_OSX
+		g_object_unref(icon);
+		icon=NULL;
+#endif
 #ifdef HAVE_NOTIFY
 		notify_uninit();
 #endif
+		gtk_widget_destroy(the_ui);
+		the_ui=NULL;
 		gdk_threads_leave();
 	}
 }
@@ -1843,14 +2119,34 @@ static gboolean on_block_termination(void){
 }
 #endif
 
-int main(int argc, char *argv[]){
-#ifdef ENABLE_NLS
-	void *p;
+static void linphone_gtk_init_ui(void){
+	linphone_gtk_init_main_window();
+
+#ifdef BUILD_WIZARD
+	// Veryfing if at least one sip account is configured. If not, show wizard
+	if (linphone_core_get_proxy_config_list(linphone_gtk_get_core()) == NULL) {
+		linphone_gtk_show_assistant();
+	}
 #endif
+
+#ifndef HAVE_GTK_OSX
+	linphone_gtk_init_status_icon();
+#endif
+	if (!iconified){
+		linphone_gtk_show_main_window();
+		linphone_gtk_check_soundcards();
+	}
+	if (linphone_gtk_get_ui_config_int("update_check_menu",0)==0)
+		linphone_gtk_check_for_new_version();
+	linphone_gtk_monitor_usb();
+}
+
+int main(int argc, char *argv[]){
 	char *config_file;
 	const char *factory_config_file;
 	const char *lang;
 	GtkSettings *settings;
+	const char *icon_path=LINPHONE_ICON;
 	GdkPixbuf *pbuf;
 	const char *app_name="Linphone";
 	LpConfig *factory;
@@ -1896,10 +2192,11 @@ int main(int argc, char *argv[]){
 	}
 
 #ifdef ENABLE_NLS
-	p=bindtextdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
-	if (p==NULL) perror("bindtextdomain failed");
+	bindtextdomain(GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
-	textdomain (GETTEXT_PACKAGE);
+	setlocale(LC_ALL,"");
+	/*do not use textdomain(): this sets a global default domain. On Mac OS bundle, it breaks gtk translations (obscure bug somewhere)*/
+	/*textdomain (GETTEXT_PACKAGE);*/
 #else
 	g_message("NLS disabled.\n");
 #endif
@@ -1913,6 +2210,14 @@ int main(int argc, char *argv[]){
 		gdk_threads_leave();
 		return -1;
 	}
+	if (config_file) free(config_file);
+	if (custom_config_file && !g_path_is_absolute(custom_config_file)) {
+		gchar *res = g_get_current_dir();
+		res = g_strjoin(G_DIR_SEPARATOR_S, res, custom_config_file, NULL);
+		free(custom_config_file);
+		custom_config_file = res;
+	}
+	config_file=linphone_gtk_get_config_file(custom_config_file);
 
 	settings=gtk_settings_get_default();
 	g_type_class_unref (g_type_class_ref (GTK_TYPE_IMAGE_MENU_ITEM));
@@ -1925,7 +2230,18 @@ int main(int argc, char *argv[]){
 			g_error("Could not change directory to %s : %s",workingdir,strerror(errno));
 		}
 	}
-
+	
+#if defined(__APPLE__) && defined(ENABLE_NLS)
+	/*workaround for bundles. GTK is unable to find translations in the bundle (obscure bug again).
+	So we help it:*/
+	{
+		if (g_file_test(PACKAGE_LOCALE_DIR, G_FILE_TEST_IS_DIR)){
+			bindtextdomain("gtk20",PACKAGE_LOCALE_DIR);
+			bindtextdomain("gdk-pixbuf",PACKAGE_LOCALE_DIR);
+			bindtextdomain("glib20",PACKAGE_LOCALE_DIR);
+		}
+	}
+#endif
 	/* Now, look for the factory configuration file, we do it this late
 		 since we want to have had time to change directory and to parse
 		 the options, in case we needed to access the working directory */
@@ -1934,14 +2250,11 @@ int main(int argc, char *argv[]){
 		factory=lp_config_new(NULL);
 		lp_config_read_file(factory,factory_config_file);
 		app_name=lp_config_get_string(factory,"GtkUi","title","Linphone");
+		icon_path=lp_config_get_string(factory,"GtkUi","icon",LINPHONE_ICON);
 	}
-
-	if (linphone_gtk_init_instance(app_name, addr_to_call) == FALSE){
-		g_warning("Another running instance of linphone has been detected. It has been woken-up.");
-		g_warning("This instance is going to exit now.");
-		gdk_threads_leave();
-		return 0;
-	}
+	g_set_application_name(app_name);
+	pbuf=create_pixbuf(icon_path);
+	if (pbuf!=NULL) gtk_window_set_default_icon(pbuf);
 
 	add_pixmap_directory("pixmaps");
 	add_pixmap_directory(PACKAGE_DATA_DIR "/pixmaps/linphone");
@@ -1954,48 +2267,40 @@ int main(int argc, char *argv[]){
 	g_signal_connect(G_OBJECT(theMacApp),"NSApplicationBlockTermination",(GCallback)on_block_termination,NULL);
 #endif
 
+core_start:
+	if (linphone_gtk_init_instance(app_name, addr_to_call) == FALSE){
+		g_warning("Another running instance of linphone has been detected. It has been woken-up.");
+		g_warning("This instance is going to exit now.");
+		gdk_threads_leave();
+		return 0;
+	}
+
 	the_ui=linphone_gtk_create_window("main");
 
 	g_object_set_data(G_OBJECT(the_ui),"is_created",GINT_TO_POINTER(FALSE));
 
 	linphone_gtk_create_log_window();
 	linphone_core_enable_logs_with_cb(linphone_gtk_log_handler);
-
+	
 	db_file=linphone_gtk_message_storage_get_db_file(NULL);
+
 	linphone_gtk_init_liblinphone(config_file, factory_config_file, db_file);
-
-	g_set_application_name(app_name);
-	pbuf=create_pixbuf(linphone_gtk_get_ui_config("icon",LINPHONE_ICON));
-	if (pbuf!=NULL) gtk_window_set_default_icon(pbuf);
-
+	
 	/* do not lower timeouts under 30 ms because it exhibits a bug on gtk+/win32, with cpu running 20% all the time...*/
 	gtk_timeout_add(30,(GtkFunction)linphone_gtk_iterate,(gpointer)linphone_gtk_get_core());
-	gtk_timeout_add(30,(GtkFunction)linphone_gtk_check_logs,(gpointer)NULL);
-	linphone_gtk_init_main_window();
-
-#ifdef BUILD_WIZARD
-	// Veryfing if at least one sip account is configured. If not, show wizard
-	if (linphone_core_get_proxy_config_list(linphone_gtk_get_core()) == NULL) {
-		linphone_gtk_show_assistant();
-	}
-#endif
-
-#ifndef HAVE_GTK_OSX
-	linphone_gtk_init_status_icon();
-#endif
-	if (!iconified){
-		linphone_gtk_show_main_window();
-		linphone_gtk_check_soundcards();
-	}
-	if (linphone_gtk_get_ui_config_int("update_check_menu",0)==0)
-		linphone_gtk_check_for_new_version();
-	linphone_gtk_monitor_usb();
-
+	gtk_timeout_add(30,(GtkFunction)linphone_gtk_check_logs,(gpointer)linphone_gtk_get_core());
+	
 	gtk_main();
 	linphone_gtk_quit();
+	
+	if (restart){
+		quit_done=FALSE;
+		restart=FALSE;
+		goto core_start;
+	}
 #ifndef HAVE_GTK_OSX
 	/*workaround a bug on win32 that makes status icon still present in the systray even after program exit.*/
-	gtk_status_icon_set_visible(icon,FALSE);
+	if (icon) gtk_status_icon_set_visible(icon,FALSE);
 #endif
 	free(progpath);
 	return 0;
