@@ -26,7 +26,9 @@
 #include "AudioRecord.h"
 #include "String8.h"
 
-#include "android_echo.h"
+#include <jni.h>
+
+#include "audiofilters/devices.h"
 
 #define NATIVE_USE_HARDWARE_RATE 1
 //#define TRACE_SND_WRITE_TIMINGS
@@ -50,13 +52,17 @@ static int std_sample_rates[]={
 };
 
 struct AndroidNativeSndCardData{
-	AndroidNativeSndCardData(): mVoipMode(0) ,mIoHandle(0){
+	AndroidNativeSndCardData(int forced_rate, audio_source_t capture_source): mVoipMode(0) ,mIoHandle(0),mCaptureSource(capture_source){
 		/* try to use the same sampling rate as the playback.*/
 		int hwrate;
 		enableVoipMode();
 		if (AudioSystem::getOutputSamplingRate(&hwrate,AUDIO_STREAM_VOICE_CALL)==0){
 			ms_message("Hardware output sampling rate is %i",hwrate);
 		}
+		if (forced_rate){
+			ms_message("Hardware is known to have bugs at default sampling rate, using %i Hz instead.",forced_rate);
+			hwrate=forced_rate;
+		}
 		mPlayRate=mRecRate=hwrate;
 		for(int i=0;;){
 			int stdrate=std_sample_rates[i];
@@ -115,6 +121,7 @@ struct AndroidNativeSndCardData{
 	int mRecRate;
 	int mRecFrames;
 	audio_io_handle_t mIoHandle;
+	audio_source_t mCaptureSource;
 };
 
 struct AndroidSndReadData{
@@ -127,18 +134,21 @@ struct AndroidSndReadData{
 		nbufs=0;
 		audio_source=AUDIO_SOURCE_DEFAULT;
 		mTickerSynchronizer=NULL;
+		aec=NULL;
 	}
 	~AndroidSndReadData(){
 		ms_mutex_destroy(&mutex);
 		flushq(&q,0);
-		delete rec;
+		if (rec) delete rec;
+		rec=0;
 	}
-	void setCard(AndroidNativeSndCardData *card){
-		mCard=card;
+	void setCard(MSSndCard *card){
+		mCard=(AndroidNativeSndCardData*)card->data;
 #ifdef NATIVE_USE_HARDWARE_RATE
-		rate=card->mRecRate;
-		rec_buf_size=card->mRecFrames * 4;
+		rate=mCard->mRecRate;
 #endif
+		rec_buf_size=mCard->mRecFrames * 4;
+		builtin_aec=card->capabilities & MS_SND_CARD_CAP_BUILTIN_ECHO_CANCELLER;
 	}
 	MSFilter *mFilter;
 	AndroidNativeSndCardData *mCard;
@@ -153,7 +163,9 @@ struct AndroidSndReadData{
 	MSTickerSynchronizer *mTickerSynchronizer;
 	int64_t read_samples;
 	audio_io_handle_t iohandle;
+	jobject aec;
 	bool started;
+	bool builtin_aec;
 };
 
 struct AndroidSndWriteData{
@@ -191,7 +203,7 @@ struct AndroidSndWriteData{
 
 static MSFilter *android_snd_card_create_reader(MSSndCard *card){
 	MSFilter *f=ms_android_snd_read_new();
-	(static_cast<AndroidSndReadData*>(f->data))->setCard((AndroidNativeSndCardData*)card->data);
+	(static_cast<AndroidSndReadData*>(f->data))->setCard(card);
 	return f;
 }
 
@@ -202,23 +214,32 @@ static MSFilter *android_snd_card_create_writer(MSSndCard *card){
 }
 
 static void android_snd_card_detect(MSSndCardManager *m){
-	if (!libmedia) libmedia=Library::load("/system/lib/libmedia.so");
-	if (!libutils) libutils=Library::load("/system/lib/libutils.so");
+	bool audio_record_loaded=false;
+	bool audio_track_loaded=false;
+	bool audio_system_loaded=false;
+	bool string8_loaded=false;
+
+	/* libmedia and libutils static variable may survive to Linphone restarts
+	 It is then necessary to perform the *::init() calls even if the libmedia and libutils are there.*/
+	if (!libmedia)
+		libmedia=Library::load("/system/lib/libmedia.so");
+	if (!libutils)
+		libutils=Library::load("/system/lib/libutils.so");
+	
 	if (libmedia && libutils){
 		/*perform initializations in order rather than in a if statement so that all missing symbols are shown in logs*/
-		bool audio_record_loaded=AudioRecordImpl::init(libmedia);
-		bool audio_track_loaded=AudioTrackImpl::init(libmedia);
-		bool audio_system_loaded=AudioSystemImpl::init(libmedia);
-		bool string8_loaded=String8Impl::init(libutils);
-		if (audio_record_loaded && audio_track_loaded && audio_system_loaded && string8_loaded){
-			ms_message("Native android sound support available.");
-			MSSndCard *card=android_snd_card_new();
-			ms_snd_card_manager_add_card(m,card);
-			return;
-		}
+		audio_record_loaded=AudioRecordImpl::init(libmedia);
+		audio_track_loaded=AudioTrackImpl::init(libmedia);
+		audio_system_loaded=AudioSystemImpl::init(libmedia);
+		string8_loaded=String8Impl::init(libutils);
+	}
+	if (audio_record_loaded && audio_track_loaded && audio_system_loaded && string8_loaded){
+		ms_message("Native android sound support available.");
+		MSSndCard *card=android_snd_card_new();
+		ms_snd_card_manager_add_card(m,card);
+		return;
 	}
 	ms_message("Native android sound support is NOT available.");
-
 }
 
 static int sdk_version = 0;
@@ -230,10 +251,12 @@ static void android_native_snd_card_init(MSSndCard *card) {
 	jfieldID fid = jni_env->GetStaticFieldID(version_class, "SDK_INT", "I");
 	sdk_version = jni_env->GetStaticIntField(version_class, fid);
 	ms_message("SDK version [%i] detected", sdk_version);
+	jni_env->DeleteLocalRef(version_class);
 }
 
 static void android_native_snd_card_uninit(MSSndCard *card){
 	
+	ms_warning("Deletion of [%p]",card->data);
 	delete static_cast<AndroidNativeSndCardData*>(card->data);
 }
 
@@ -256,19 +279,16 @@ MSSndCardDesc android_native_snd_card_desc={
 static MSSndCard * android_snd_card_new(void)
 {
 	MSSndCard * obj;
-	EchoCancellerParams params;
+	SoundDeviceDescription *d;
 	
 	obj=ms_snd_card_new(&android_native_snd_card_desc);
 	obj->name=ms_strdup("android sound card");
-	obj->data=new AndroidNativeSndCardData();
 	
-	if (android_sound_get_echo_params(&params)==0){
-		if (params.has_builtin_ec) obj->capabilities|=MS_SND_CARD_CAP_BUILTIN_ECHO_CANCELLER;
-		else obj->latency=params.delay;
-	}else{
-		obj->latency=250;
-		ms_warning("Model not echo-calibrated, using default delay value: %i ms",obj->latency); 
-	}
+	d=sound_device_description_get();
+	if (d->flags & DEVICE_HAS_BUILTIN_AEC) obj->capabilities|=MS_SND_CARD_CAP_BUILTIN_ECHO_CANCELLER;
+	obj->latency=d->delay;
+	obj->data = new AndroidNativeSndCardData(	d->recommended_rate
+												,(d->flags & DEVICE_USE_ANDROID_MIC) ?AUDIO_SOURCE_MIC:AUDIO_SOURCE_VOICE_COMMUNICATION);
 	return obj;
 }
 
@@ -299,12 +319,13 @@ static void android_snd_read_cb(int event, void* user, void *p_info){
 		ms_ticker_set_time_func(obj->ticker,(uint64_t (*)(void*))ms_ticker_synchronizer_get_corrected_time, ad->mTickerSynchronizer);
 	}
 	if (event==AudioRecord::EVENT_MORE_DATA){
-		AudioRecord::Buffer * info=reinterpret_cast<AudioRecord::Buffer*>(p_info);
-		if (info->size > 0) {
-			mblk_t *m=allocb(info->size,0);
-			memcpy(m->b_wptr,info->raw,info->size);
-			m->b_wptr+=info->size;
-			ad->read_samples+=info->frameCount;
+		AudioRecord::Buffer info;
+		AudioRecord::readBuffer(p_info,&info);
+		if (info.size > 0) {
+			mblk_t *m=allocb(info.size,0);
+			memcpy(m->b_wptr,info.raw,info.size);
+			m->b_wptr+=info.size;
+			ad->read_samples+=info.frameCount;
 
 			ms_mutex_lock(&ad->mutex);
 			compute_timespec(ad);
@@ -317,6 +338,61 @@ static void android_snd_read_cb(int event, void* user, void *p_info){
 	}
 }
 
+static void android_snd_read_activate_hardware_aec(MSFilter *obj){
+	AndroidSndReadData *ad=(AndroidSndReadData*)obj->data;
+	JNIEnv *env=ms_get_jni_env();
+	int sessionId=ad->rec->getSessionId();
+	
+	if (sessionId==-1) return;
+	
+	jclass aecClass = env->FindClass("android/media/audiofx/AcousticEchoCanceler");
+	if (aecClass==NULL){
+		env->ExceptionClear(); //very important.
+		return;
+	}
+	//aecClass= (jclass)env->NewGlobalRef(aecClass);
+	jmethodID isAvailableID = env->GetStaticMethodID(aecClass,"isAvailable","()Z");
+	if (isAvailableID!=NULL){
+		jboolean ret=env->CallStaticBooleanMethod(aecClass,isAvailableID);
+		if (ret){
+			jmethodID createID = env->GetStaticMethodID(aecClass,"create","(I)Landroid/media/audiofx/AcousticEchoCanceler;");
+			if (createID!=NULL){
+				ad->aec=env->CallStaticObjectMethod(aecClass,createID,sessionId);
+				if (ad->aec){
+					ad->aec=env->NewGlobalRef(ad->aec);
+					ms_message("AcousticEchoCanceler successfully created.");
+					jclass effectClass=env->FindClass("android/media/audiofx/AudioEffect");
+					if (effectClass){
+						//effectClass=(jclass)env->NewGlobalRef(effectClass);
+						jmethodID isEnabledID = env->GetMethodID(effectClass,"getEnabled","()Z");
+						jmethodID setEnabledID = env->GetMethodID(effectClass,"setEnabled","(Z)I");
+						if (isEnabledID && setEnabledID){
+							jboolean enabled=env->CallBooleanMethod(ad->aec,isEnabledID);
+							ms_message("AcousticEchoCanceler enabled: %i",(int)enabled);
+							if (!enabled){
+								int ret=env->CallIntMethod(ad->aec,setEnabledID,TRUE);
+								if (ret!=0){
+									ms_error("Could not enable AcousticEchoCanceler: %i",ret);
+								}
+							}
+						}
+						env->DeleteLocalRef(effectClass);
+					}
+				}else{
+					ms_error("Failed to create AcousticEchoCanceler.");
+				}
+			}else{
+				ms_error("create() not found in class AcousticEchoCanceler !");
+				env->ExceptionClear(); //very important.
+			}
+		}
+	}else{
+		ms_error("isAvailable() not found in class AcousticEchoCanceler !");
+		env->ExceptionClear(); //very important.
+	}
+	env->DeleteLocalRef(aecClass);
+}
+
 static void android_snd_read_preprocess(MSFilter *obj){
 	AndroidSndReadData *ad=(AndroidSndReadData*)obj->data;
 	status_t  ss;
@@ -327,18 +403,19 @@ static void android_snd_read_preprocess(MSFilter *obj){
 	ad->mFilter=obj;
 	ad->read_samples=0;
 	ad->started=FALSE;
-	ad->audio_source=AUDIO_SOURCE_VOICE_COMMUNICATION;
+	ad->audio_source=ad->mCard->mCaptureSource; /*some device require to capture from MIC instead of from voice communications*/
 	for(int i=0;i<2;i++){
 		ad->rec=new AudioRecord(ad->audio_source,
 						ad->rate,
 						AUDIO_FORMAT_PCM_16_BIT,
 						audio_channel_in_mask_from_count(ad->nchannels),
 						ad->rec_buf_size,
-						(AudioRecord::record_flags)0 /*flags ??*/
-						,android_snd_read_cb,ad,notify_frames,0);
+						android_snd_read_cb,ad,notify_frames,0);
 		ss=ad->rec->initCheck();
+		ms_message("Setting up AudioRecord  source=%i,rate=%i,framecount=%i",ad->audio_source,ad->rate,ad->rec_buf_size);
+
 		if (ss!=0){
-			ms_error("Problem when setting up AudioRecord:%s  source=%i,rate=%i,framecount=%i",strerror(-ss),ad->audio_source,ad->rate,ad->rec_buf_size);
+			ms_error("Problem when setting up AudioRecord:%s ",strerror(-ss));
 			delete ad->rec;
 			ad->rec=0;
 			if (i == 0) {
@@ -349,9 +426,9 @@ static void android_snd_read_preprocess(MSFilter *obj){
 	}
 
 	if (ad->rec != 0) {
+		if (ad->builtin_aec) android_snd_read_activate_hardware_aec(obj);
 		ad->rec->start();
 	}
-
 }
 
 static void android_snd_read_postprocess(MSFilter *obj){
@@ -360,6 +437,11 @@ static void android_snd_read_postprocess(MSFilter *obj){
 	if (ad->rec!=0) {
 		ad->started=false;
 		ad->rec->stop();
+		if (ad->aec){
+			JNIEnv *env=ms_get_jni_env();
+			env->DeleteGlobalRef(ad->aec);
+			ad->aec=NULL;
+		}
 		delete ad->rec;
 		ad->rec=0;
 	}
@@ -535,12 +617,14 @@ static void android_snd_write_cb(int event, void *user, void * p_info){
 	AndroidSndWriteData *ad=(AndroidSndWriteData*)user;
 	
 	if (event==AudioTrack::EVENT_MORE_DATA){
-		AudioTrack::Buffer *info=reinterpret_cast<AudioTrack::Buffer *>(p_info);
+		AudioTrack::Buffer info;
 		int avail;
 		int ask;
+		
+		AudioTrack::readBuffer(p_info,&info);
 
 		ms_mutex_lock(&ad->mutex);
-		ask = info->size;
+		ask = info.size;
 		avail = ms_bufferizer_get_avail(&ad->bf);
 		/* Drop the samples accumulated before the first callback asking for data. */
 		if ((ad->nbufs == 0) && (avail > (ask * 2))) {
@@ -553,25 +637,25 @@ static void android_snd_write_cb(int event, void *user, void * p_info){
 				ad->minBufferFilling = avail;
 			}
 		}
-		info->size = MIN(avail, ask);
+		info.size = MIN(avail, ask);
+		info.frameCount = info.size / 2;
 #ifdef TRACE_SND_WRITE_TIMINGS
 		{
 			MSTimeSpec ts;
 			ms_get_cur_time(&ts);
 			ms_message("%03u.%03u: AudioTrack ask %d, given %d, available %d [%f ms]", (uint32_t) ts.tv_sec, (uint32_t) (ts.tv_nsec / 1000),
-				ask, info->size, avail, (avail / (2 * ad->nchannels)) / (ad->rate / 1000.0));
+				ask, info.size, avail, (avail / (2 * ad->nchannels)) / (ad->rate / 1000.0));
 		}
 #endif
-		if (info->size > 0){
-			ms_bufferizer_read(&ad->bf,(uint8_t*)info->raw,info->size);
-			info->frameCount = info->size / 2;
+		if (info.size > 0){
+			ms_bufferizer_read(&ad->bf,(uint8_t*)info.raw,info.size);
 		}else{
 			/* we have an underrun (no more samples to deliver to the callback). We need to reset minBufferFilling*/
 			ad->minBufferFilling=-1;
 		}
 		ms_mutex_unlock(&ad->mutex);
 		ad->nbufs++;
-		ad->nFramesRequested+=info->frameCount;
+		ad->nFramesRequested+=info.frameCount;
 		/*
 		if (ad->nbufs %100){
 			uint32_t pos;
@@ -580,6 +664,7 @@ static void android_snd_write_cb(int event, void *user, void * p_info){
 			}
 		}
 		*/
+		AudioTrack::writeBuffer(p_info,&info);
 	}else if (event==AudioTrack::EVENT_UNDERRUN){
 		ms_mutex_lock(&ad->mutex);
 #ifdef TRACE_SND_WRITE_TIMINGS
@@ -631,7 +716,7 @@ static void android_snd_write_preprocess(MSFilter *obj){
                      channel_mask_for_audio_track(ad->nchannels),
                      play_buf_size,
                      AUDIO_OUTPUT_FLAG_NONE, // AUDIO_OUTPUT_FLAG_NONE,
-                     android_snd_write_cb, ad,notify_frames,0);
+                     android_snd_write_cb, ad,notify_frames,0, AudioTrack::TRANSFER_CALLBACK);
 	s=ad->tr->initCheck();
 	if (s!=0) {
 		ms_error("Problem setting up AudioTrack: %s",strerror(-s));
@@ -695,9 +780,13 @@ static void android_snd_write_postprocess(MSFilter *obj){
 	if (!ad->tr) return;
 	ms_message("Stopping sound playback");
 	ad->tr->stop();
-	ad->tr->flush();
+	while(!ad->tr->stopped()){
+		usleep(20000);
+	}
 	ms_message("Sound playback stopped");
-	delete ad->tr;
+	ad->tr->flush();
+	ms_message("Sound playback flushed, deleting");
+	if (ad->tr) delete ad->tr;
 	ad->tr=NULL;
 	ad->mCard->disableVoipMode();
 	ad->mStarted=false;
