@@ -74,11 +74,11 @@ static void event_cb(void *ud, MSFilter* f, unsigned int event, void *eventdata)
 	}
 }
 
-static void video_steam_process_rtcp(VideoStream *stream, mblk_t *m){
+static void video_stream_process_rtcp(MediaStream *media_stream, mblk_t *m){
+	VideoStream *stream = (VideoStream *)media_stream;
 	do{
 		if (rtcp_is_SR(m)){
 			const report_block_t *rb;
-			ms_message("video_steam_process_rtcp: receiving RTCP SR");
 			rb=rtcp_SR_get_report_block(m,0);
 			if (rb){
 				unsigned int ij;
@@ -86,7 +86,7 @@ static void video_steam_process_rtcp(VideoStream *stream, mblk_t *m){
 				float flost;
 				ij=report_block_get_interarrival_jitter(rb);
 				flost=(float)(100.0*report_block_get_fraction_lost(rb)/256.0);
-				ms_message("video_steam_process_rtcp: interarrival jitter=%u , lost packets percentage since last report=%f, round trip time=%f seconds",ij,flost,rt);
+				ms_message("video_stream_process_rtcp[%p]: interarrival jitter=%u , lost packets percentage since last report=%f, round trip time=%f seconds",stream,ij,flost,rt);
 				if (stream->ms.rc)
 					ms_bitrate_controller_process_rtcp(stream->ms.rc,m);
 			}
@@ -104,24 +104,6 @@ static void stop_preload_graph(VideoStream *stream){
 }
 
 void video_stream_iterate(VideoStream *stream){
-	/*
-	if (stream->output!=NULL)
-		ms_filter_call_method_noarg(stream->output,
-			MS_VIDEO_OUT_HANDLE_RESIZING);
-	*/
-	if (stream->ms.evq){
-		OrtpEvent *ev;
-		while (NULL != (ev=ortp_ev_queue_get(stream->ms.evq))) {
-			OrtpEventType evt=ortp_event_get_type(ev);
-			if (evt == ORTP_EVENT_RTCP_PACKET_RECEIVED){
-				OrtpEventData *evd=ortp_event_get_data(ev);
-				video_steam_process_rtcp(stream,evd->packet);
-			}else if ((evt == ORTP_EVENT_STUN_PACKET_RECEIVED) && (stream->ms.ice_check_list)) {
-				ice_handle_stun_packet(stream->ms.ice_check_list,stream->ms.session,ortp_event_get_data(ev));
-			}
-			ortp_event_destroy(ev);
-		}
-	}
 	media_stream_iterate(&stream->ms);
 }
 
@@ -132,10 +114,10 @@ static void choose_display_name(VideoStream *stream){
 	stream->display_name=ms_strdup("MSAndroidDisplay");
 #elif __APPLE__ && !defined(__ios)
 	stream->display_name=ms_strdup("MSOSXGLDisplay");
-#elif defined(HAVE_GL)
-	stream->display_name=ms_strdup("MSGLXVideo");
 #elif defined (HAVE_XV)
 	stream->display_name=ms_strdup("MSX11Video");
+#elif defined(HAVE_GL)
+	stream->display_name=ms_strdup("MSGLXVideo");
 #elif defined(__ios)
 	stream->display_name=ms_strdup("IOSDisplay");	
 #else
@@ -152,14 +134,13 @@ VideoStream *video_stream_new(int loc_rtp_port, int loc_rtcp_port, bool_t use_ip
 	stream->ms.rtpsend=ms_filter_new(MS_RTP_SEND_ID);
 	stream->ms.ice_check_list=NULL;
 	rtp_session_register_event_queue(stream->ms.session,stream->ms.evq);
-	stream->sent_vsize.width=MS_VIDEO_SIZE_CIF_W;
-	stream->sent_vsize.height=MS_VIDEO_SIZE_CIF_H;
+	MS_VIDEO_SIZE_ASSIGN(stream->sent_vsize, CIF);
 	stream->dir=VideoStreamSendRecv;
 	stream->display_filter_auto_rotate_enabled=0;
 	stream->source_performs_encoding = FALSE;
 	stream->output_performs_decoding = FALSE;
 	choose_display_name(stream);
-
+	stream->ms.process_rtcp=video_stream_process_rtcp;
 	return stream;
 }
 
@@ -168,6 +149,24 @@ void video_stream_set_sent_video_size(VideoStream *stream, MSVideoSize vsize){
 	stream->sent_vsize=vsize;
 }
 
+MSVideoSize video_stream_get_sent_video_size(const VideoStream *stream) {
+	MSVideoSize vsize;
+	MS_VIDEO_SIZE_ASSIGN(vsize, UNKNOWN);
+	if (stream->ms.encoder != NULL) {
+		ms_filter_call_method(stream->ms.encoder, MS_FILTER_GET_VIDEO_SIZE, &vsize);
+	}
+	return vsize;
+}
+
+MSVideoSize video_stream_get_received_video_size(const VideoStream *stream) {
+	MSVideoSize vsize;
+	MS_VIDEO_SIZE_ASSIGN(vsize, UNKNOWN);
+	if (stream->ms.decoder != NULL) {
+		ms_filter_call_method(stream->ms.decoder, MS_FILTER_GET_VIDEO_SIZE, &vsize);
+	}
+	return vsize;
+}
+
 void video_stream_set_relay_session_id(VideoStream *stream, const char *id){
 	ms_filter_call_method(stream->ms.rtpsend, MS_RTP_SEND_SET_RELAY_SESSION_ID,(void*)id);
 }
@@ -241,8 +240,11 @@ static void configure_video_source(VideoStream *stream){
 
 	/* transmit orientation to source filter */
 	ms_filter_call_method(stream->source,MS_VIDEO_CAPTURE_SET_DEVICE_ORIENTATION,&stream->device_orientation);
-	
-    /* transmit its preview window id if any to source filter*/
+	/* initialize the capture device orientation for preview */
+	if( ms_filter_has_method(stream->source, MS_VIDEO_DISPLAY_SET_DEVICE_ORIENTATION) )
+		ms_filter_call_method(stream->source,MS_VIDEO_DISPLAY_SET_DEVICE_ORIENTATION,&stream->device_orientation);
+
+	/* transmit its preview window id if any to source filter*/
 	if (stream->preview_window_id!=0){
 		video_stream_set_native_preview_window_id(stream, stream->preview_window_id);
 	}
@@ -327,7 +329,8 @@ int video_stream_start (VideoStream *stream, RtpProfile *profile, const char *re
 	}
 
 	rtp_session_set_profile(rtps,profile);
-	if (rem_rtp_port>0) rtp_session_set_remote_addr_full(rtps,rem_rtp_ip,rem_rtp_port,rem_rtcp_ip,rem_rtcp_port);
+	if (rem_rtp_port>0)
+		rtp_session_set_remote_addr_full(rtps,rem_rtp_ip,rem_rtp_port,rem_rtcp_ip,rem_rtcp_port);
 	rtp_session_set_payload_type(rtps,payload);
 	rtp_session_set_jitter_compensation(rtps,jitt_comp);
 
@@ -347,7 +350,7 @@ int video_stream_start (VideoStream *stream, RtpProfile *profile, const char *re
 		if (rem_rtp_port>0) ms_filter_call_method(stream->ms.rtpsend,MS_RTP_SEND_SET_SESSION,stream->ms.session);
 		if (stream->source_performs_encoding == FALSE) {
 			stream->ms.encoder=ms_filter_create_encoder(pt->mime_type);
-			if ((stream->ms.encoder==NULL) ){
+			if (stream->ms.encoder==NULL){
 				/* big problem: we don't have a registered codec for this payload...*/
 				ms_error("videostream.c: No encoder available for payload %i:%s.",payload,pt->mime_type);
 				return -1;
@@ -362,8 +365,15 @@ int video_stream_start (VideoStream *stream, RtpProfile *profile, const char *re
 		}
 
 		if (pt->normal_bitrate>0){
+			MSVideoConfiguration *vconf_list = NULL;
 			ms_message("Limiting bitrate of video encoder to %i bits/s",pt->normal_bitrate);
-			ms_filter_call_method(stream->ms.encoder,MS_FILTER_SET_BITRATE,&pt->normal_bitrate);
+			ms_filter_call_method(stream->ms.encoder, MS_VIDEO_ENCODER_GET_CONFIGURATION_LIST, &vconf_list);
+			if (vconf_list != NULL) {
+				MSVideoConfiguration vconf = ms_video_find_best_configuration_for_bitrate(vconf_list, pt->normal_bitrate);
+				ms_filter_call_method(stream->ms.encoder, MS_VIDEO_ENCODER_SET_CONFIGURATION, &vconf);
+			} else {
+				ms_filter_call_method(stream->ms.encoder, MS_FILTER_SET_BITRATE, &pt->normal_bitrate);
+			}
 		}
 		if (pt->send_fmtp){
 			ms_filter_call_method(stream->ms.encoder,MS_FILTER_ADD_FMTP,pt->send_fmtp);
@@ -423,7 +433,7 @@ int video_stream_start (VideoStream *stream, RtpProfile *profile, const char *re
 			stream->ms.decoder = stream->output;	/* Consider the decoder is the output */
 		} else {
 			stream->ms.decoder=ms_filter_create_decoder(pt->mime_type);
-			if ((stream->ms.decoder==NULL) ){
+			if (stream->ms.decoder==NULL){
 				/* big problem: we don't have a registered decoderfor this payload...*/
 				ms_error("videostream.c: No decoder available for payload %i:%s.",payload,pt->mime_type);
 				ms_filter_destroy(stream->output);
@@ -716,8 +726,7 @@ int video_stream_get_camera_sensor_rotation(VideoStream *stream) {
 
 VideoPreview * video_preview_new(void){
 	VideoPreview *stream = (VideoPreview *)ms_new0 (VideoPreview, 1);
-	stream->sent_vsize.width=MS_VIDEO_SIZE_CIF_W;
-	stream->sent_vsize.height=MS_VIDEO_SIZE_CIF_H;
+	MS_VIDEO_SIZE_ASSIGN(stream->sent_vsize, CIF);
 	choose_display_name(stream);
 	return stream;
 }
